////////////////////////////////////////////////////////////////////////////////////////////////////
// Histograms
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <iomanip>
#include <string>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <fstream>
#include <sstream>
#include <sys/stat.h>
#include "TChain.h"
#include <TApplication.h>
#include <TCanvas.h>
#include <TMultiGraph.h>
#include <TGraph.h>
#include <TFile.h>
#include <TH1D.h>
#include <TH2D.h>
#include <TLegend.h>
#include <TPaveText.h>
#include <TLorentzVector.h>
#include <TStyle.h>
#include <TTree.h>
#include <TTreeReader.h>
#include <TColor.h>
#define SP12 setprecision(12) << setw(21)
#define SW5 setw(9)

using namespace std;

const double PI = 3.1415926535897932385;

void set_style();

double Observable(int iObs, double *px, double *py, double *pz, double *E);

int main()
{
// Create output folder
    mkdir("Results/histograms", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
//--------------------------------------------------------------------------------------------------
    set_style();
    TCanvas c("c", "c", 3000, 2000);
    const string htitle("");
    c.SetLeftMargin(0.14);
    c.SetRightMargin(0.1);
    c.SetTopMargin(0.12);
    c.SetBottomMargin(0.15);
//--------------------------------------------------------------------------------------------------
// User defined colors
    TColor kColor0(10000, 0, 0, 0);
    TColor kColor1(10001, 1, 0, 0);
    TColor kColor2(10002, 0, 1, 0);
    TColor kColor3(10003, 0, 0, 1);
//--------------------------------------------------------------------------------------------------
    double born_xs_dif = 0.0;
    double loop_xs_dif = 0.0;
    double born_xs_sum = 0.0;
    double loop_xs_sum = 0.0;

    double weightborn[1], pxborn[3], pyborn[3], pzborn[3], Eborn[3];
    double weightloop[1], pxloop[3], pyloop[3], pzloop[3], Eloop[3];

    const int NOBS = 6;
    const int LNAME = 50;

    double ObsMin[NOBS],ObsMax[NOBS];
    double ObsBornMin[NOBS],ObsBornMax[NOBS];
    double ObsLoopMin[NOBS],ObsLoopMax[NOBS];

    ofstream fObs[NOBS];
    char obsName[NOBS][LNAME];
    char xTitle[NOBS][LNAME];
    char nameRoot[NOBS][LNAME];
    char namePdf[NOBS][LNAME];
    char nameDat[NOBS][LNAME];

    const int    binN[NOBS] = { 100, 100, 100, 100,  100,  100};
    const double binL[NOBS] = {  0., -1., -2., -4.,   0.,   0.};
    const double binU[NOBS] = {180.,  1.,  2.,  4., 250., 250.};

    sprintf(obsName[0], "Theta3");
    sprintf(xTitle[0], "#theta(3) [deg.]");
    sprintf(obsName[1], "Cost3");
    sprintf(xTitle[1], "cos#theta(3)");
    sprintf(obsName[2], "Y3");
    sprintf(xTitle[2], "Y(3)");
    sprintf(obsName[3], "Eta3");
    sprintf(xTitle[3], "#eta(3)");
    sprintf(obsName[4], "Pt3");
    sprintf(xTitle[4], "P_{T}(3) [GeV]");
    sprintf(obsName[5], "M34");
    sprintf(xTitle[5], "M(34) [GeV]");

    double binW[NOBS];
    for (int i = 0; i < NOBS; i++) {
	binW[i] = (binU[i]-binL[i])/binN[i];
    }

    for (int i = 0; i < NOBS; i++) {
	sprintf(nameRoot[i], "Results/histograms/%sALR.root", obsName[i]);
	sprintf(namePdf[i], "Results/histograms/%sALR.pdf", obsName[i]);
	sprintf(nameDat[i], "Results/histograms/%sALR.dat", obsName[i]);
    }
//--------------------------------------------------------------------------------------------------
    TFile *fborndif = new TFile("Results/SANCeeLOdif.root");
    TTree *tborndif = (TTree*)fborndif->Get("events_born");
    tborndif->SetBranchAddress("weight_born",&weightborn);
    tborndif->SetBranchAddress("px_born",&pxborn);
    tborndif->SetBranchAddress("py_born",&pyborn);
    tborndif->SetBranchAddress("pz_born",&pzborn);
    tborndif->SetBranchAddress("E_born",&Eborn);
//--------------------------------------------------------------------------------------------------
    TFile *floopdif = new TFile("Results/SANCeeNLOdif.root");
    TTree *tloopdif = (TTree*)floopdif->Get("events_tot");
    tloopdif->SetBranchAddress("weight_tot",&weightloop);
    tloopdif->SetBranchAddress("px_tot",&pxloop);
    tloopdif->SetBranchAddress("py_tot",&pyloop);
    tloopdif->SetBranchAddress("pz_tot",&pzloop);
    tloopdif->SetBranchAddress("E_tot",&Eloop);
//--------------------------------------------------------------------------------------------------
    TFile *fbornsum = new TFile("Results/SANCeeLOsum.root");
    TTree *tbornsum = (TTree*)fbornsum->Get("events_born");
    tbornsum->SetBranchAddress("weight_born",&weightborn);
    tbornsum->SetBranchAddress("px_born",&pxborn);
    tbornsum->SetBranchAddress("py_born",&pyborn);
    tbornsum->SetBranchAddress("pz_born",&pzborn);
    tbornsum->SetBranchAddress("E_born",&Eborn);
//--------------------------------------------------------------------------------------------------
    TFile *floopsum = new TFile("Results/SANCeeNLOsum.root");
    TTree *tloopsum = (TTree*)floopsum->Get("events_tot");
    tloopsum->SetBranchAddress("weight_tot",&weightloop);
    tloopsum->SetBranchAddress("px_tot",&pxloop);
    tloopsum->SetBranchAddress("py_tot",&pyloop);
    tloopsum->SetBranchAddress("pz_tot",&pzloop);
    tloopsum->SetBranchAddress("E_tot",&Eloop);
//--------------------------------------------------------------------------------------------------
    ifstream f_born_xs_dif;
    f_born_xs_dif.open("Results/SANCeeLOdif.dat");
    f_born_xs_dif >> born_xs_dif;
    f_born_xs_dif.close();
    born_xs_dif *= 1.E3;
//--------------------------------------------------------------------------------------------------
    ifstream f_loop_xs_dif;
    f_loop_xs_dif.open("Results/SANCeeNLOdif.dat");
    f_loop_xs_dif >> loop_xs_dif;
    f_loop_xs_dif.close();
    loop_xs_dif *= 1.E3;
//--------------------------------------------------------------------------------------------------
    ifstream f_born_xs_sum;
    f_born_xs_sum.open("Results/SANCeeLOsum.dat");
    f_born_xs_sum >> born_xs_sum;
    f_born_xs_sum.close();
    born_xs_sum *= 1.E3;
//--------------------------------------------------------------------------------------------------
    ifstream f_loop_xs_sum;
    f_loop_xs_sum.open("Results/SANCeeNLOsum.dat");
    f_loop_xs_sum >> loop_xs_sum;
    f_loop_xs_sum.close();
    loop_xs_sum *= 1.E3;
//--------------------------------------------------------------------------------------------------
    int nentriesborndif = tborndif->GetEntries();
    int nentriesloopdif = tloopdif->GetEntries();
    int nentriesbornsum = tbornsum->GetEntries();
    int nentriesloopsum = tloopsum->GetEntries();
//--------------------------------------------------------------------------------------------------
    TH1D *hBornDif[NOBS], *hBornSum[NOBS];
    TH1D *hLoopDif[NOBS], *hLoopSum[NOBS];
    TH1D *hBornAlr[NOBS], *hBornAlrMin[NOBS], *hBornAlrMax[NOBS];
    TH1D *hLoopAlr[NOBS], *hLoopAlrMin[NOBS], *hLoopAlrMax[NOBS];
//--------------------------------------------------------------------------------------------------
    for (int i = 0; i < NOBS; i++) {
	hBornDif[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hBornDif[i]->Sumw2();
	hBornSum[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hBornSum[i]->Sumw2();

	hLoopDif[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hLoopDif[i]->Sumw2();
	hLoopSum[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hLoopSum[i]->Sumw2();

	gStyle->SetHistLineColor(10000);
	gStyle->SetHistFillColor(10000);
	gStyle->SetMarkerColor(10000);
	gStyle->SetHistLineWidth(1);

	hBornAlr[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hBornAlr[i]->Sumw2();
	hBornAlrMin[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hBornAlrMin[i]->Sumw2();
	hBornAlrMax[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hBornAlrMax[i]->Sumw2();

	gStyle->SetHistLineColor(10001);
	gStyle->SetHistFillColor(10001);
	gStyle->SetMarkerColor(10001);
	gStyle->SetHistLineWidth(1);

	hLoopAlr[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hLoopAlr[i]->Sumw2();
	hLoopAlrMin[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hLoopAlrMin[i]->Sumw2();
	hLoopAlrMax[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hLoopAlrMax[i]->Sumw2();

    }
//--------------------------------------------------------------------------------------------------
    for (int i = 0; i < nentriesborndif; i++) {
	tborndif->GetEntry(i);
	for (int j = 0; j < NOBS; j++) {
	    hBornDif[j]->Fill(Observable(j, pxborn, pyborn, pzborn, Eborn), weightborn[0]*born_xs_dif/nentriesborndif/binW[j]);
	}
    }

    for (int i = 0; i < nentriesloopdif; i++) {
	tloopdif->GetEntry(i);
	for (int j = 0; j < NOBS; j++) {
	    hLoopDif[j]->Fill(Observable(j, pxloop, pyloop, pzloop, Eloop), weightloop[0]*loop_xs_dif/nentriesloopdif/binW[j]);
	}
    }

    for (int i = 0; i < nentriesbornsum; i++) {
	tbornsum->GetEntry(i);
	for (int j = 0; j < NOBS; j++) {
	    hBornSum[j]->Fill(Observable(j, pxborn, pyborn, pzborn, Eborn), weightborn[0]*born_xs_sum/nentriesbornsum/binW[j]);
	}
    }

    for (int i = 0; i < nentriesloopsum; i++) {
	tloopsum->GetEntry(i);
	for (int j = 0; j < NOBS; j++) {
	    hLoopSum[j]->Fill(Observable(j, pxloop, pyloop, pzloop, Eloop), weightloop[0]*loop_xs_sum/nentriesloopsum/binW[j]);
	}
    }

    for (int i = 0; i < NOBS; i++) {
	hBornAlr[i]->Divide(hBornDif[i],hBornSum[i],1.,1.);
	hLoopAlr[i]->Divide(hLoopDif[i],hLoopSum[i],1.,1.);
    }

    for (int i = 0; i < NOBS; i++) {
	for (int j = 1; j <= binN[i]; j++) {
	    hBornAlrMin[i]->SetBinContent(j, hBornAlr[i]->GetBinContent(j)-hBornAlr[i]->GetBinError(j));
	    hBornAlrMax[i]->SetBinContent(j, hBornAlr[i]->GetBinContent(j)+hBornAlr[i]->GetBinError(j));
	    hLoopAlrMin[i]->SetBinContent(j, hLoopAlr[i]->GetBinContent(j)-hLoopAlr[i]->GetBinError(j));
	    hLoopAlrMax[i]->SetBinContent(j, hLoopAlr[i]->GetBinContent(j)+hLoopAlr[i]->GetBinError(j));
	}
	ObsBornMin[i] = hBornAlrMin[i]->GetBinContent(hBornAlrMin[i]->GetMinimumBin());
	ObsBornMax[i] = hBornAlrMax[i]->GetBinContent(hBornAlrMax[i]->GetMaximumBin());
	ObsLoopMin[i] = hLoopAlrMin[i]->GetBinContent(hLoopAlrMin[i]->GetMinimumBin());
	ObsLoopMax[i] = hLoopAlrMax[i]->GetBinContent(hLoopAlrMax[i]->GetMaximumBin());
	ObsMin[i] = min(ObsBornMin[i], ObsLoopMin[i]);
	ObsMax[i] = max(ObsBornMax[i], ObsLoopMax[i]);
    }
//--------------------------------------------------------------------------------------------------
    TLegend *legend = new TLegend(0.15, 0.9, 0.45, 1.0);
    legend->SetFillColor(0);
    legend->SetFillStyle(4000);
    legend->SetBorderSize(0);
    legend->AddEntry(hBornAlr[0], "Born", "L");
    legend->AddEntry(hLoopAlr[0], "1-loop", "L");
//--------------------------------------------------------------------------------------------------
    for (int i = 0; i < NOBS; i++) {
	c.SetLogy(0);
	hBornAlr[i]->SetMinimum(ObsMin[i]);
	hBornAlr[i]->SetMaximum(ObsMax[i]);
	hBornAlr[i]->GetXaxis()->SetTitle(xTitle[i]);
	hBornAlr[i]->GetYaxis()->SetTitle("A_{LR}");
	hBornAlr[i]->DrawCopy("");
	hBornAlr[i]->DrawCopy("hist same");
	hLoopAlr[i]->DrawCopy("same ");
	hLoopAlr[i]->DrawCopy("hist same");
	legend->Draw();
	c.Update();
	c.SaveAs(nameRoot[i]);
	c.Print(namePdf[i]);

	fObs[i].open(nameDat[i]);
	fObs[i] << "Born:" << endl;
	for (int j = 1; j <= binN[i]; j++) {
	    fObs[i] << SW5 << binL[i]+binW[i]*(j-0.5) << SP12 << hBornAlr[i]->GetBinContent(j) << SP12 << hBornAlr[i]->GetBinError(j) << endl;
	}
	fObs[i] << "1-loop:" << endl;
	for (int j = 1; j <= binN[i]; j++) {
	    fObs[i] << SW5 << binL[i]+binW[i]*(j-0.5) << SP12 << hLoopAlr[i]->GetBinContent(j) << SP12 << hLoopAlr[i]->GetBinError(j) << endl;
	}
	fObs[i].close();
    }

    for (int i = 0; i < NOBS; i++) {
	delete hBornDif[i];
	delete hLoopDif[i];
	delete hBornSum[i];
	delete hLoopSum[i];
	delete hBornAlr[i];
	delete hBornAlrMin[i];
	delete hBornAlrMax[i];
	delete hLoopAlr[i];
	delete hLoopAlrMin[i];
	delete hLoopAlrMax[i];
    }

    return 0;
}

void set_style()
{
    gStyle->SetCanvasColor(kWhite);
    gStyle->SetHistFillColor(kWhite);
    gStyle->SetHistLineColor(kBlack);
    gStyle->SetHistFillStyle(0);
    gStyle->SetFillStyle(4050);
    gStyle->SetMarkerColor(kBlack);
    gStyle->SetMarkerStyle(1);
    gStyle->SetHistLineStyle(1);
    gStyle->SetHistLineWidth(1);
    gStyle->SetPaperSize(20, 28);
    gStyle->SetPalette(1);
    gStyle->SetOptStat(0);
    gStyle->SetLabelFont(42,"X");
    gStyle->SetLabelSize(0.06,"X");
    gStyle->SetLabelOffset(0.006,"X");
    gStyle->SetTitleFont(42,"X");
    gStyle->SetTitleSize(0.06,"X");
    gStyle->SetTitleOffset(1.1,"X");
    gStyle->SetTitleColor(kBlack,"X");
    gStyle->SetLabelFont(42,"Y");
    gStyle->SetLabelSize(0.06,"Y");
    gStyle->SetLabelOffset(0.006,"Y");
    gStyle->SetTitleFont(42,"Y");
    gStyle->SetTitleSize(0.06,"Y");
    gStyle->SetTitleOffset(1.1,"Y");
    gStyle->SetTitleColor(kBlack,"Y");
    gStyle->SetTextFont(42);
}

double Observable(int iObs, double *px, double *py, double *pz, double *E) {
    double Obs;
    switch (iObs) {
    case 0: // Angle theta for 3rd particle
	Obs = 180/PI*acos(pz[0]/sqrt(px[0]*px[0]+py[0]*py[0]+pz[0]*pz[0]));
	break;
    case 1: // Cosine of theta for 3rd particle
	Obs = pz[0]/sqrt(px[0]*px[0]+py[0]*py[0]+pz[0]*pz[0]);
	break;
    case 2: // Rapidity of 3rd particle
	Obs = 0.5*log((E[0]+pz[0])/(E[0]-pz[0]));
	break;
    case 3: // Pseudorapidity of 3rd particle
	Obs = 0.5*log((sqrt(px[0]*px[0]+py[0]*py[0]+pz[0]*pz[0])+pz[0])/(sqrt(px[0]*px[0]+py[0]*py[0]+pz[0]*pz[0])-pz[0]));
	break;
    case 4: // Transverse momentum of 3rd particle
	Obs = sqrt(px[0]*px[0]+py[0]*py[0]);
    case 5: // Invariant mass of 3rd and 4th particles
	Obs = sqrt((E[0]+E[1])*(E[0]+E[1])-(px[0]+px[1])*(px[0]+px[1])-(py[0]+py[1])*(py[0]+py[1])-(pz[0]+pz[1])*(pz[0]+pz[1]));
	break;
    default:
	Obs = 0.;
    }
    return Obs;
};
