#ifndef INFO_H
#define INFO_H
#include "General.h"

namespace SANC {

class Settings;

class Info
{
public:
    Info(Settings & _cfg);
    std::string version();
    void init();
/*
    Process ID:
    101: e+e- -> e-e+
    102: e+e- -> ZH
    103: e+e- -> mu-mu+
    104: e+e- -> tau-tau+
*/
    const int Pid;
    //--------------------------------------------------------------------------------------------------
    const double LamMinus; // Longitudinal polarization of electron beam
    const double LamPlus; // Longitudinal polarization of positron beam
    //--------------------------------------------------------------------------------------------------
    const int Scheme; // Scheme: 0 - alpha(0), 1 - Gmu, 2 - alpha(M_Z)
    double Ecm; // Total energy of colliding particles in CM frame
    const double Ome;
    const double DeltaLL;
    const double ScaleLL;
    const double Alphai;
    const double AlphaMZi;
    //--------------------------------------------------------------------------------------------------
    const int Alr;
    //--------------------------------------------------------------------------------------------------
    const int Iborn;
    const int Iqed;
    const int Iqedll;
    const int Iew;
    const int Ilin;
    const int Ifgg;
    const int Irun;
    //--------------------------------------------------------------------------------------------------
    const bool ForceOffH;
    const bool ForceOffSV;
    //--------------------------------------------------------------------------------------------------
    const double Men;
    const double Mel;
    const double Mup;
    const double Mdn;
    const double Mmn;
    const double Mmo;
    const double Mch;
    const double Mst;
    const double Mtn;
    const double Mta;
    const double Mtp;
    const double Mbt;
    const double MW;
    const double MZ;
    const double MH;
    const double Wz;
    const double Ww;
    const double Wh;
    const double Wtp;
    //--------------------------------------------------------------------------------------------------
    const double CutMinCth;
    const double CutMaxCth;
    const double CutMinTheta;
    const double CutMaxTheta;
    double CutMinM34;
    double CutMaxM34;
    double CutMinPt;
    double CutMaxPt;
    double CutMaxPhotonE;
    //--------------------------------------------------------------------------
    const long RanSeed; // Seed for RNG initialization (0 <= RanSeed <= 9E8)
    const long NevBorn; // MC statistics for Born
    const long NevVirt; // MC statistics for Virt
    const long NevHard; // MC statistics for Hard
    const long NevLL11; // MC statistics for LL11
    const long NevLL22; // MC statistics for LL22
    const long NevLL12; // MC statistics for LL12
    const long NevTot; // MC statistics for Total
    //--------------------------------------------------------------------------------------------------
    //  FOAM parameters for Born
    const int  nCellsBorn;
    const int  nSamplBorn;
    const int  nBinBorn;
    const int  EvPerBinBorn;
    const double MaxWtRejBorn;
    //--------------------------------------------------------------------------------------------------
    //  FOAM parameters for Virt
    const int  nCellsVirt;
    const int  nSamplVirt;
    const int  nBinVirt;
    const int  EvPerBinVirt;
    const double MaxWtRejVirt;
    //--------------------------------------------------------------------------------------------------
    //  FOAM parameters for Hard
    const int  nCellsHard; // Number of Cells
    const int  nSamplHard; // Number of MC events per cell in build-up
    const int  nBinHard; // Number of bins in build-up
    const int  EvPerBinHard; // Maximum events (equiv.) per bin in buid-up
    const double MaxWtRejHard; // Maximum wt for rejection, for OptRej=1
    //--------------------------------------------------------------------------------------------------
    //  FOAM parameters for LL11
    const int  nCellsLL11;
    const int  nSamplLL11;
    const int  nBinLL11;
    const int  EvPerBinLL11;
    const double MaxWtRejLL11;
    //--------------------------------------------------------------------------------------------------
    //  FOAM parameters for LL22
    const int  nCellsLL22;
    const int  nSamplLL22;
    const int  nBinLL22;
    const int  EvPerBinLL22;
    const double MaxWtRejLL22;
    //--------------------------------------------------------------------------------------------------
    //  FOAM parameters for LL12
    const int  nCellsLL12;
    const int  nSamplLL12;
    const int  nBinLL12;
    const int  EvPerBinLL12;
    const double MaxWtRejLL12;
    //--------------------------------------------------------------------------------------------------
    //  General FOAM parameters
    const int  OptRej; // =0, weighted events;  =1, wt=1 events
    const int  OptDrive; // (D=2) Option, type of Drive =0,1,2 for TrueVol,Sigma,WtMax
    const int  Chat; // Chat level
    //--------------------------------------------------------------------------------------------------
    //  Parameters for branching
    double SigmaBorn;
    double SigmaVirt;
    double SigmaHard;
    double SigmaLL11;
    double SigmaLL22;
    double SigmaLL12;
    double SigmaTot;
    //--------------------------------------------------------------------------------------------------
    const bool IEXPLBORN;
    const bool IEXPLVIRT;
    const bool IEXPLHARD;
    const bool IEXPLLL11;
    const bool IEXPLLL22;
    const bool IEXPLLL12;
    //--------------------------------------------------------------------------------------------------
    const bool PrintLHE;
    const bool PrintROOT;
    const bool PrintHard;
    //--------------------------------------------------------------------------------------------------
    const int verMajor,verMinor,verPatch;
    std::string outputDir;

private:
    Settings & cfg;
};

}
#endif // INFO_H
