      subroutine isrlla_factor (ktype,s,x1,x2,flla)
      implicit none!
      include 's2n_declare.h'
      integer ktype
      real*8 x1,x2,flla
      real*8 ll,llmo,ll1a,ll2a,ll2p,ll3a,ll3p,ll4a
      real*8 flla1a,flla2a,flla2p,flla3a,flla3p,flla4a
      real*8 ald
      real*8 P10,P20,P30,P40
      real*8 P1,P2,P3,P4
      real*8 RS,RP

      flla = 0d0

c  ktype defines the type of kinematics
c  ktype = 1: Born x Born kinematics (sigma_{ISR LLA} ~ flla * Born(s))
c  ktype = 2: Hard x Born kinematics (sigma_{ISR LLA} ~ flla * Born(x1*s))
c  ktype = 3: Born x Hard kinematics (sigma_{ISR LLA} ~ flla * Born(x2*s))
c  ktype = 4: Hard x Hard kinematics (sigma_{ISR LLA} ~ flla * Born(x1*x2*s))

      if ((ktype.lt.1).or.(ktype.gt.4)) return
      if ((iqedll.lt.1).or.(iqedll.gt.11)) return

c  default value for deltall is 10^-4
      ald = log(deltall)
      ll = log(s/mel**2)
      llmo = log(s/mmo**2)
      ll1a = (alpha/2d0/pi)*(ll-1)
      ll2a = (alpha/2d0/pi)**2*(ll-1)**2
      ll2p = (alpha/2d0/pi)**2*ll**2
      ll3a = (alpha/2d0/pi)**3*(ll-1)**3
      ll3p = (alpha/2d0/pi)**3*ll**3
      ll4a = (alpha/2d0/pi)**4*(ll-1)**4

      if (ktype.eq.1) then
        flla1a = ll1a*(2d0*P10(ald))
        flla2a = ll2a*(P20(ald)+P10(ald)*P10(ald))
        flla2p = ll2p*(2d0/3d0*P10(ald))
        flla3a = ll3a*(1d0/3d0*P30(ald)+P10(ald)*P20(ald))
        flla3p = ll3p*(2d0/3d0*P20(ald)+8d0/27d0*P10(ald)+2d0/3d0*P10(ald)*P10(ald))
        flla4a = ll4a*(1d0/12d0*P40(ald)+1d0/3d0*P10(ald)*P30(ald)+1d0/4d0*(P20(ald))**2)
      elseif (ktype.eq.2) then
        flla1a = ll1a*P1(x1)
        flla2a = ll2a*(1d0/2d0*P2(x1)+P1(x1)*P10(ald))
        flla2p = ll2p*(1d0/3d0*P1(x1)+1d0/2d0*RS(x1))
        flla3a = ll3a*(1d0/6d0*P3(x1)+1d0/2d0*P2(x1)*P10(ald)+1d0/2d0*P1(x1)*P20(ald))
        flla3p = ll3p*(1d0/3d0*P2(x1)+4d0/27d0*P1(x1)+1d0/3d0*RP(x1)-1d0/9*RS(x1)+2d0/3d0*P1(x1)*P10(ald)+1d0/2d0*RS(x1)*P10(ald))
        flla4a = ll4a*(1d0/24d0*P4(x1)+1d0/6d0*P3(x1)*P10(ald)+1d0/4d0*P2(x1)*P20(ald)+1d0/6d0*P1(x1)*P30(ald))
      elseif (ktype.eq.3) then
        flla1a = ll1a*P1(x2)
        flla2a = ll2a*(1d0/2d0*P2(x2)+P1(x2)*P10(ald))
        flla2p = ll2p*(1d0/3d0*P1(x2)+1d0/2d0*RS(x2))
        flla3a = ll3a*(1d0/6d0*P3(x2)+1d0/2d0*P2(x2)*P10(ald)+1d0/2d0*P1(x2)*P20(ald))
        flla3p = ll3p*(1d0/3d0*P2(x2)+4d0/27d0*P1(x2)+1d0/3d0*RP(x2)-1d0/9*RS(x2)+2d0/3d0*P1(x2)*P10(ald)+1d0/2d0*RS(x2)*P10(ald))
        flla4a = ll4a*(1d0/24d0*P4(x2)+1d0/6d0*P3(x2)*P10(ald)+1d0/4d0*P2(x2)*P20(ald)+1d0/6d0*P1(x2)*P30(ald))
      elseif (ktype.eq.4) then
        flla2a = ll2a*P1(x1)*P1(x2)
        flla2p = 0d0
        flla3a = ll3a*(1d0/2d0*P2(x1)*P1(x2)+1d0/2d0*P1(x1)*P2(x2))
        flla3p = ll3p*(2d0/3d0*P1(x1)*P1(x2)+1d0/2d0*RS(x1)*P1(x2)+1d0/2d0*P1(x1)*RS(x2))
        flla4a = ll4a*(1d0/4d0*P2(x1)*P2(x2)+1d0/6d0*P1(x1)*P3(x2)+1d0/6d0*P3(x1)*P1(x2))
      endif


      if (iqedll.eq.1) then
c       O(alpha L) gamma:
        flla = flla1a
      elseif (iqedll.eq.2) then
c       O(alpha^2 L^2) gamma:
        flla = flla2a
      elseif (iqedll.eq.3) then
c       O(alpha^2 L^2) el pair:
        flla = flla2p
      elseif (iqedll.eq.4) then
c       O(alpha^2 L^2) mu pair:
        flla = flla2p*llmo/ll
      elseif (iqedll.eq.5) then
c       O(alpha^3 L^3) gamma:
        flla = flla3a
      elseif (iqedll.eq.6) then
c       O(alpha^3 L^3) el pair:
        flla = flla3p
      elseif (iqedll.eq.7) then
c       O(alpha^3 L^3) mu pair:
        flla = flla3p*llmo/ll
      elseif (iqedll.eq.8) then
c       O(alpha^4 L^4) gamma:
        flla = flla4a
      elseif (iqedll.eq.9) then
c       sum_{n=2}^2 O(alpha^n L^n):
        flla = flla2a+flla2p*(1.+llmo/ll)
      elseif (iqedll.eq.10) then
c       sum_{n=2}^3 O(alpha^n L^n):
        flla = flla2a+flla2p*(1.+llmo/ll)+flla3a+flla3p*(1.+llmo/ll)
      elseif (iqedll.eq.11) then
c       sum_{n=2}^4 O(alpha^n L^n):
        flla = flla2a+flla2p*(1.+llmo/ll)+flla3a+flla3p*(1.+llmo/ll)+flla4a
      endif

      return
      end
