************************************************************************
* sanc_nc_v2.00 package.
************************************************************************
* File (nc_br_12_12__16_16.f) is created on Сб янв 27 22:51:07 MSK 2018.
************************************************************************
* This is the FORTRAN module (nc_br_12_12__16_16) to calculate EW Born
* cross section and soft photon Bremsstrahlung
* for the el^- + el^- -> mo^- + mo^- process.
*
* It is produced by s2n package of SANC (v1.10) Project.
* Copyright (C), SANC Project Team, 2002-2021.
*
* http://sanc.jinr.ru/
* E-mail: <sanc@jinr.ru>; <sanc-team@cern.ch>
************************************************************************
      subroutine nc_br_12_12__16_16 (qs,ts,us,born,soft,hard)
      implicit none!
      include 's2n_declare.h'

      integer iborn_
      real*8 lnlaom
      real*8 softisr,softfsr,softifi
      real*8 betasmomo,ibetasmomo
      real*8 jintqsmomo, lpblmbmomo

      soft = 0d0
      hard = 0d0

      cmz2 = mz2

      s = -qs
      t = -ts
      u = -us

      betasmomo = sqrt(1d0-4d0*mmo**2/s)
      ibetasmomo = 1d0/betasmomo

      iborn_ = iborn
      iborn  = 1
      call nc_si_12_12__16_16 (s,t,u,born)
      iborn  = iborn_
      if (born.ne.born) print *,'Born NAN!!!',s,t,u

      if (iborn.eq.1) return

      lnlaom = log(4d0*omega**2/tlmu2)
      lpblmbmomo=s*(1+betasmomo)**2/4d0/mmo**2
      jintqsmomo = log(lpblmbmomo)

      softisr=
     &  -born/pi*alpha*qel**2*(
     &  lnlaom-log(s/mel**2)
     &  -log(s/mel**2)*lnlaom
     &  +log(s/mel**2)**2/2d0+pi**2/3d0
     & )

      softfsr=
     &  -born/pi*alpha*qta**2*(
     &  lnlaom+ibetasmomo*(-jintqsmomo
     &  -(1d0+betasmomo**2)/2d0*(
     &  jintqsmomo*lnlaom
     &  +2d0*ddilog(1d0-lpblmbmomo)
     &  +jintqsmomo**2/2d0
     & )))

      softifi=
     & -2d0*qel*qta*alpha/pi*born * (
     & +log((mmo**2+us)/(mmo**2+ts))*lnlaom
     & -ddilog(1d0-s*(1d0+betasmomo)/2d0/(mmo**2+us))
     & -ddilog(1d0-2d0*mmo**2/(mmo**2+us)/(1d0+betasmomo))
     & +ddilog(1d0-s*(1d0+betasmomo)/2d0/(mmo**2+ts))
     & +ddilog(1d0-2d0*mmo**2/(mmo**2+ts)/(1d0+betasmomo))
     &  );

      if (iqed.eq.1) then
         soft = softisr+softfsr+softifi
      elseif(iqed.eq.2) then
         soft = softisr
      elseif(iqed.eq.3) then
         soft = softifi
      elseif(iqed.eq.4) then
         soft = softfsr
      elseif(iqed.eq.5) then
         soft = softifi+softfsr
      elseif(iqed.eq.6) then
         soft = softisr+softifi
      endif
      soft = soft*cfprime
      if (soft.ne.soft) then
         print *,'Soft NAN!!!',s,t,u,born,softisr,softfsr,softifi
      endif
      return
      end
