#include "Info.h"
#include "Settings.h"

using namespace SANC;

Info::Info(Settings & _cfg):
    Pid(_cfg.get_i("pid")),
    LamMinus(_cfg.get_d("lamem")),
    LamPlus(_cfg.get_d("lamep")),
    Scheme(_cfg.get_i("gfscheme")),
    Ecm(_cfg.get_d("ecm")),
    Ome(_cfg.get_d("ome")),
    DeltaLL(_cfg.get_d("deltall")),
    ScaleLL(_cfg.get_d("scalell")),
    Alphai(_cfg.get_d("alphai")),
    AlphaMZi(_cfg.get_d("alphamzi")),
    Alr(_cfg.get_i("alr")),
    Iborn(_cfg.get_i("iborn")),
    Iqed(_cfg.get_i("iqed")),
    Iqedll(_cfg.get_i("iqedll")),
    Iew(_cfg.get_i("iew")),
    Ilin(_cfg.get_i("ilin")),
    Ifgg(_cfg.get_i("ifgg")),
    Irun(_cfg.get_i("irun")),
    ForceOffH(_cfg.get_b("forceOffH")),
    ForceOffSV(_cfg.get_b("forceOffSV")),
    Men(_cfg.get_d("men")),
    Mel(_cfg.get_d("mel")),
    Mup(_cfg.get_d("mup")),
    Mdn(_cfg.get_d("mdn")),
    Mmn(_cfg.get_d("mmn")),
    Mmo(_cfg.get_d("mmo")),
    Mch(_cfg.get_d("mch")),
    Mst(_cfg.get_d("mst")),
    Mtn(_cfg.get_d("mtn")),
    Mta(_cfg.get_d("mta")),
    Mtp(_cfg.get_d("mtp")),
    Mbt(_cfg.get_d("mbt")),
    MW(_cfg.get_d("mw")),
    MZ(_cfg.get_d("mz")),
    MH(_cfg.get_d("mh")),
    Wz(_cfg.get_d("wz")),
    Ww(_cfg.get_d("ww")),
    Wh(_cfg.get_d("wh")),
    Wtp(_cfg.get_d("wtp")),
    CutMinCth(-_cfg.get_d("costhcut")),
    CutMaxCth(_cfg.get_d("costhcut")),
    CutMinTheta(_cfg.get_d("thetacut")),
    CutMaxTheta(180.-_cfg.get_d("thetacut")),
    CutMinM34(0.),
    CutMaxM34(Ecm),
    CutMinPt(0.),
    CutMaxPt(Ecm),
    CutMaxPhotonE(_cfg.get_d("e5max")),
    RanSeed(_cfg.get_l("RanSeed")),
    NevBorn(_cfg.get_l("NevBorn")),
    NevVirt(_cfg.get_l("NevVirt")),
    NevHard(_cfg.get_l("NevHard")),
    NevLL11(_cfg.get_l("NevLL11")),
    NevLL22(_cfg.get_l("NevLL22")),
    NevLL12(_cfg.get_l("NevLL12")),
    NevTot(_cfg.get_l("NevTot")),
    nCellsBorn(_cfg.get_i("nCellsBorn")),
    nSamplBorn(_cfg.get_i("nSamplBorn")),
    nBinBorn(_cfg.get_i("nBinBorn")),
    EvPerBinBorn(_cfg.get_i("EvPerBinBorn")),
    MaxWtRejBorn(_cfg.get_d("MaxWtRejBorn")),
    nCellsVirt(_cfg.get_i("nCellsVirt")),
    nSamplVirt(_cfg.get_i("nSamplVirt")),
    nBinVirt(_cfg.get_i("nBinVirt")),
    EvPerBinVirt(_cfg.get_i("EvPerBinVirt")),
    MaxWtRejVirt(_cfg.get_d("MaxWtRejVirt")),
    nCellsHard(_cfg.get_i("nCellsHard")),
    nSamplHard(_cfg.get_i("nSamplHard")),
    nBinHard(_cfg.get_i("nBinHard")),
    EvPerBinHard(_cfg.get_i("EvPerBinHard")),
    MaxWtRejHard(_cfg.get_d("MaxWtRejHard")),
    nCellsLL11(_cfg.get_i("nCellsLL11")),
    nSamplLL11(_cfg.get_i("nSamplLL11")),
    nBinLL11(_cfg.get_i("nBinLL11")),
    EvPerBinLL11(_cfg.get_i("EvPerBinLL11")),
    MaxWtRejLL11(_cfg.get_d("MaxWtRejLL11")),
    nCellsLL22(_cfg.get_i("nCellsLL22")),
    nSamplLL22(_cfg.get_i("nSamplLL22")),
    nBinLL22(_cfg.get_i("nBinLL22")),
    EvPerBinLL22(_cfg.get_i("EvPerBinLL22")),
    MaxWtRejLL22(_cfg.get_d("MaxWtRejLL22")),
    nCellsLL12(_cfg.get_i("nCellsLL12")),
    nSamplLL12(_cfg.get_i("nSamplLL12")),
    nBinLL12(_cfg.get_i("nBinLL12")),
    EvPerBinLL12(_cfg.get_i("EvPerBinLL12")),
    MaxWtRejLL12(_cfg.get_d("MaxWtRejLL12")),
    OptRej(_cfg.get_i("OptRej")),
    OptDrive(_cfg.get_i("OptDrive")),
    Chat(_cfg.get_i("Chat")),
    IEXPLBORN(_cfg.get_b("exploreBorn")),
    IEXPLVIRT(_cfg.get_b("exploreVirt")),
    IEXPLHARD(_cfg.get_b("exploreHard")),
    IEXPLLL11(_cfg.get_b("exploreLL11")),
    IEXPLLL22(_cfg.get_b("exploreLL22")),
    IEXPLLL12(_cfg.get_b("exploreLL12")),
    PrintLHE(_cfg.get_b("printLHE")),
    PrintROOT(_cfg.get_b("printROOT")),
    PrintHard(_cfg.get_b("printHard")),
    verMajor(1),
    verMinor(2),
    verPatch(1),
    cfg(_cfg)
{
    outputDir = cfg.get_s("outputDir");
    Ecm = cfg.get_d("ecm");
    CutMinM34 = 0.;
    CutMaxM34 = Ecm;
    CutMinPt = 0.;
    CutMaxPt = Ecm;

}

std::string Info::version() {
    return std::to_string(verMajor)+"."+std::to_string(verMinor)+"."+std::to_string(verPatch);
}

void Info::init() {
    Ecm = cfg.get_d("ecm");
}
