#include <cmath>
#include "General.h"
#include "Integrand.h"
#include "Info.h"
#include "LHEF.h"
#include "ModulesInterface.h"
#include "Process.h"
#include "Vec4.h"
using namespace std;
using namespace SANC;
using namespace SANCmodules;
using namespace SANCmodulesVar;

Integrand::Integrand(SANC::Info &_info, SANC::Process & _proc, int type):
info(_info),proc(_proc)
{
    m_Type=type;
    Mf1 = proc.Mf1;
    Mf2 = proc.Mf2;
    Mf3 = proc.Mf3;
    Mf4 = proc.Mf4;
    Mf1_2 = proc.Mf1_2;
    Mf2_2 = proc.Mf2_2;
    Mf3_2 = proc.Mf3_2;
    Mf4_2 = proc.Mf4_2;
    sm = proc.sm;
    dm = proc.dm;
}

double Integrand::Density(int nDim, double *Xarg) {
    double rho;
    rho = 1e-100;
    if (m_Type == 0) rho = Born(nDim, Xarg);
    else if (m_Type == 1) rho = Virt(nDim, Xarg);
    else if (m_Type == 2) rho = Hard(nDim, Xarg);
    else if (m_Type == 3) rho = Loop(nDim, Xarg);
    else if (m_Type == 4) rho = LL11(nDim, Xarg);
    else if (m_Type == 5) rho = LL22(nDim, Xarg);
    else if (m_Type == 6) rho = LL12(nDim, Xarg);
    else{
	cout << " TFDISTR::Density: !!!!!! WRONG m_Type= " << m_Type << endl;
    }
    return rho;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand::Born(int nDim, double *Xarg) {
    double BornXS(0.);
    double Xl[2],Xu[2],Z[2];
    double Factor = 1.;
    double Cth3,Phi3;
    double Jac;
    double Sth3,Cph3,Sph3;
    double beta12;
    double Pf3,Enf3,Enf4;
    double S,Qs,Qt,Qu,Tn,Un;
    double Cth13,Cth14,Theta13,Theta14,Pt3,Pt4;
    double Born(0.),Soft(0.),Hard(0.);
    Vec4   P[6];
    proc.event.weightProc = 1.;
    if ((info.Ecm<Mf1+Mf2)||(info.Ecm<Mf3+Mf4)) return 0.;
//--------------------------------------------------------------------------------------------------
    Xl[0] = -1.;
    Xu[0] = 1.;
    Xl[1] = 0.;
    Xu[1] = 2.*consts_.pi;
    for (int i = 0; i < 2; i++) {
	Z[i] = Xl[i]+(Xu[i]-Xl[i])*Xarg[i];
	Factor *= Xu[i]-Xl[i];
    }
//--------------------------------------------------------------------------------------------------
    Cth3 = Z[0];
    Phi3 = Z[1];
//--------------------------------------------------------------------------------------------------
    Sth3 = sqrt(1.-Cth3*Cth3);
    Cph3 = cos(Phi3);
    Sph3 = sin(Phi3);
    S = pow2(info.Ecm);
    beta12 = (proc.beta1*proc.E1-proc.beta2*proc.E2)/(proc.E1+proc.E2);
    Mf1_2 = pow2(Mf1);
    Mf2_2 = pow2(Mf2);
    Mf3_2 = pow2(Mf3);
    Mf4_2 = pow2(Mf4);
    Pf3 = sqrt((S-pow2(Mf3+Mf4))*(S-pow2(Mf3-Mf4)))/2./info.Ecm;
    Enf3 = (S+Mf3_2-Mf4_2)/2./info.Ecm;
    Enf4 = (S+Mf4_2-Mf3_2)/2./info.Ecm;
    Jac = 1./2./consts_.pi;
//--------------------------------------------------------------------------------------------------
    P[0] = Vec4();
    P[1] = Vec4(0.,0.,proc.beta1*proc.E1,proc.E1,Mf1);
    P[2] = Vec4(0.,0.,-proc.beta2*proc.E2,proc.E2,Mf2);
    P[3] = Vec4(0.,0.,Pf3,Enf3,Mf3).rot(Sth3,Cth3,Sph3,Cph3).bstZ(beta12);
    P[4] = Vec4(0.,0.,-Pf3,Enf4,Mf4).rot(Sth3,Cth3,Sph3,Cph3).bstZ(beta12);
    P[5] = Vec4();
//--------------------------------------------------------------------------------------------------
    Cth13 = costheta(P[1],P[3]);
    Cth14 = costheta(P[1],P[4]);
    Theta13 = 180./consts_.pi*acos(Cth13);
    Theta14 = 180./consts_.pi*acos(Cth14);
    Pt3 = P[3].pT();
    Pt4 = P[4].pT();
    Tn = t(P[1],P[4]);
    Un = t(P[1],P[3]);
//--------------------------------------------------------------------------------------------------
    if ((abs(Cth13) > 1.0-1.e-7) || (abs(Cth14) > 1.0-1.e-7)) return BornXS;
    if (Cth13 < info.CutMinCth) return BornXS;
    if (Cth13 > info.CutMaxCth) return BornXS;
    if (Cth14 < info.CutMinCth) return BornXS;
    if (Cth14 > info.CutMaxCth) return BornXS;
    if (Theta13 < info.CutMinTheta) return BornXS;
    if (Theta13 > info.CutMaxTheta) return BornXS;
    if (Theta14 < info.CutMinTheta) return BornXS;
    if (Theta14 > info.CutMaxTheta) return BornXS;
    if (Pt3 < info.CutMinPt) return BornXS;
    if (Pt3 > info.CutMaxPt) return BornXS;
    if (Pt4 < info.CutMinPt) return BornXS;
    if (Pt4 > info.CutMaxPt) return BornXS;
//--------------------------------------------------------------------------------------------------
    Qs = -S;
    Qt = -Tn;
    Qu = -Un;
    flags_.iborn = 0;
//--------------------------------------------------------------------------------------------------
    proc.born(&Qs,&Qt,&Qu,&Born,&Soft,&Hard);
//--------------------------------------------------------------------------------------------------
    BornXS = Born*Factor*Jac;
    if (BornXS < 0) proc.event.weightProc = -1.;
    BornXS = abs(BornXS);
//--------------------------------------------------------------------------------------------------
    proc.event.P(0)=P[0];
    proc.event.P(1)=P[1];
    proc.event.P(2)=P[2];
    proc.event.P(3)=P[3];
    proc.event.P(4)=P[4];
    proc.event.P(5)=P[5];
//--------------------------------------------------------------------------------------------------
    return BornXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand::Virt(int nDim, double *Xarg) {
    double VirtXS(0.);
    double Xl[2],Xu[2],Z[2];
    double Factor(1.);
    double Cth3,Phi3;
    double X1,X2;
    double Jac;
    double Sth3,Cph3,Sph3;
    double beta12;
    double Pf3,Enf3,Enf4;
    double S,T,U,Qs,Qt,Qu;
    double Cth13,Cth14,Theta13,Theta14,Pt3,Pt4;
    double Born(0.),Soft(0.),Hard(0.),Sigma(0.);
    Vec4   P[6];
    int    ktype(1);
    double FactorLL(0.);
    proc.event.weightProc = 1.;
    if ((info.Ecm<Mf1+Mf2)||(info.Ecm<Mf3+Mf4)) return 0.;
//--------------------------------------------------------------------------------------------------
    Xl[0] = -1.;
    Xu[0] = 1.;
    Xl[1] = 0.;
    Xu[1] = 2.*consts_.pi;
    for (int i = 0; i < 2; i++) {
	Z[i] = Xl[i]+(Xu[i]-Xl[i])*Xarg[i];
	Factor *= Xu[i]-Xl[i];
    }
//--------------------------------------------------------------------------------------------------
    Cth3 = Z[0];
    Phi3 = Z[1];
//--------------------------------------------------------------------------------------------------
    X1 = 1.;
    X2 = 1.;
//--------------------------------------------------------------------------------------------------
    Sth3 = sqrt(1.-Cth3*Cth3);
    Cph3 = cos(Phi3);
    Sph3 = sin(Phi3);
    S = pow2(info.Ecm);
    beta12 = (proc.beta1*proc.E1-proc.beta2*proc.E2)/(proc.E1+proc.E2);
    Mf1_2 = pow2(Mf1);
    Mf2_2 = pow2(Mf2);
    Mf3_2 = pow2(Mf3);
    Mf4_2 = pow2(Mf4);
    Pf3 = sqrt((S-pow2(Mf3+Mf4))*(S-pow2(Mf3-Mf4)))/2./info.Ecm;
    Enf3 = (S+Mf3_2-Mf4_2)/2./info.Ecm;
    Enf4 = (S+Mf4_2-Mf3_2)/2./info.Ecm;
    Jac = 1./2./consts_.pi;
//--------------------------------------------------------------------------------------------------
    P[0] = Vec4();
    P[1] = Vec4(0.,0.,proc.beta1*proc.E1,proc.E1,Mf1);
    P[2] = Vec4(0.,0.,-proc.beta2*proc.E2,proc.E2,Mf2);
    P[3] = Vec4(0.,0.,Pf3,Enf3,Mf3).rot(Sth3,Cth3,Sph3,Cph3).bstZ(beta12);
    P[4] = Vec4(0.,0.,-Pf3,Enf4,Mf4).rot(Sth3,Cth3,Sph3,Cph3).bstZ(beta12);
    P[5] = Vec4();
//--------------------------------------------------------------------------------------------------
    Cth13 = costheta(P[1],P[3]);
    Cth14 = costheta(P[1],P[4]);
    Theta13 = 180./consts_.pi*acos(Cth13);
    Theta14 = 180./consts_.pi*acos(Cth14);
    Pt3 = P[3].pT();
    Pt4 = P[4].pT();
    T = t(P[1],P[4]);
    U = t(P[1],P[3]);
//--------------------------------------------------------------------------------------------------
    if ((abs(Cth13) > 1.0-1.e-7) || (abs(Cth14) > 1.0-1.e-7)) return VirtXS;
    if (Cth13 < info.CutMinCth) return VirtXS;
    if (Cth13 > info.CutMaxCth) return VirtXS;
    if (Cth14 < info.CutMinCth) return VirtXS;
    if (Cth14 > info.CutMaxCth) return VirtXS;
    if (Theta13 < info.CutMinTheta) return VirtXS;
    if (Theta13 > info.CutMaxTheta) return VirtXS;
    if (Theta14 < info.CutMinTheta) return VirtXS;
    if (Theta14 > info.CutMaxTheta) return VirtXS;
    if (Pt3 < info.CutMinPt) return VirtXS;
    if (Pt3 > info.CutMaxPt) return VirtXS;
    if (Pt4 < info.CutMinPt) return VirtXS;
    if (Pt4 > info.CutMaxPt) return VirtXS;
//--------------------------------------------------------------------------------------------------
    if (info.Iqedll != 0) {
        isrlla_factor_(&ktype,&S,&X1,&X2,&FactorLL);
    }
//--------------------------------------------------------------------------------------------------
    Qs = -S;
    Qt = -T;
    Qu = -U;
    flags_.iborn = 0;
//--------------------------------------------------------------------------------------------------
    proc.born(&Qs,&Qt,&Qu,&Born,&Soft,&Hard);
    if (info.Iqed != 0 || info.Iew != 0) {
        proc.virt(&S,&T,&U,&Sigma);
    }
    if (info.Iqed == 0) {
	    Soft = 0.;
        if (info.Iew == 0) {
            Sigma = Born;
        }
    }
//--------------------------------------------------------------------------------------------------
    VirtXS = (Sigma+Soft)*Factor*Jac
        +Born*Factor*Jac*FactorLL;
    if (VirtXS < 0) proc.event.weightProc = -1.;
    VirtXS = abs(VirtXS);
//--------------------------------------------------------------------------------------------------
    proc.event.P(0)=P[0];
    proc.event.P(1)=P[1];
    proc.event.P(2)=P[2];
    proc.event.P(3)=P[3];
    proc.event.P(4)=P[4];
    proc.event.P(5)=P[5];
//--------------------------------------------------------------------------------------------------
    return VirtXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand::Hard(int nDim, double *Xarg) {
    double HardXS(0.);
    double Xl[5],Xu[5],Z[5];
    double Factor(1.);
    double Spr,Eta3,Phi3,Eta5,Phi5;
    double Jac;
    double Cth3,Sth3,Cph3,Sph3,Cth5,Sth5,Cph5,Sph5;
    double Beta34,beta12;
    double Pf3,Enf3,Enf4;
    double S,Sqspr;
    double Egamma;
    double Cth13,Cth14,Theta13,Theta14,Pt3,Pt4;
    double Hard(0.);
    Vec4 * P[6];
    P[0] = &proc.event.P(0);
    P[1] = &proc.event.P(1);
    P[2] = &proc.event.P(2);
    P[3] = &proc.event.P(3);
    P[4] = &proc.event.P(4);
    P[5] = &proc.event.P(5);
    proc.event.weightProc = 1.;
    if ((info.Ecm<Mf1+Mf2)||(info.Ecm<Mf3+Mf4)) return 0.;
//--------------------------------------------------------------------------------------------------
    Xl[0] = log(info.Ome);
    Xu[0] = log(1.-(Mf3+Mf4)*(Mf3+Mf4)/(info.Ecm*info.Ecm));
    Xl[1] = -50.;
    Xu[1] = 50.;
    Xl[2] = 0.;
    Xu[2] = 2.*consts_.pi;
    Xl[3] = -50.;
    Xu[3] = 50.;
    Xl[4] = 0.;
    Xu[4] = 2.*consts_.pi;
    for (int i = 0; i < 5; i++) {
	Z[i] = Xl[i]+(Xu[i]-Xl[i])*Xarg[i];
	Factor *= Xu[i]-Xl[i];
    }
//--------------------------------------------------------------------------------------------------
    Spr= pow2(info.Ecm)*(1.-exp(Z[0]));
    if (Spr==0.)
    {
        return HardXS;
    }
    Eta3 = Z[1];
    Phi3 = Z[2];
    Eta5 = Z[3];
    Phi5 = Z[4];
//--------------------------------------------------------------------------------------------------
    Cth3 = (exp(2.0*Eta3)-1.0)/(exp(2.0*Eta3)+1.0);
    Sth3 = 2.0*exp(Eta3)/(exp(2.0*Eta3)+1.0);
    Cph3 = cos(Phi3);
    Sph3 = sin(Phi3);
    Cth5 = (exp(2.0*Eta5)-1.0)/(exp(2.0*Eta5)+1.0);
    Sth5 = 2.0*exp(Eta5)/(exp(2.0*Eta5)+1.0);
    Cph5 = cos(Phi5);
    Sph5 = sin(Phi5);
    S = pow2(info.Ecm);
    beta12 = (proc.beta1*proc.E1-proc.beta2*proc.E2)/(proc.E1+proc.E2);
    Sqspr = sqrt(Spr);
    Mf1_2 = pow2(Mf1);
    Mf2_2 = pow2(Mf2);
    Mf3_2 = pow2(Mf3);
    Mf4_2 = pow2(Mf4);
    Pf3 = sqrt((Spr-pow2(Mf3+Mf4))*(Spr-pow2(Mf3-Mf4)))/2.0/Sqspr;
    Enf3 = (Spr+Mf3_2-Mf4_2)/2./Sqspr;
    Enf4 = (Spr+Mf4_2-Mf3_2)/2./Sqspr;
    Egamma = (S-Spr)/2./info.Ecm;
    Beta34 = (S-Spr)/(S+Spr);
    Jac = Sth3*Sth3*Sth5*Sth5/2./consts_.pi*(S-Spr);
//--------------------------------------------------------------------------------------------------
// NOTE: Because SANC use special reference frame,
// such as phi3=phi3'+pi/2 and th5 is photon`s angle,
// we need replace Sph3->-Cph3, Cph3->Sph3 and boost in opposite direction
    *P[0] = Vec4();
    *P[1] = Vec4(0.,0.,proc.beta1*proc.E1,proc.E1,Mf1);
    *P[2] = Vec4(0.,0.,-proc.beta2*proc.E2,proc.E2,Mf2);
    *P[3] = Vec4(0.,0.,Pf3,Enf3,Mf3).rot(Sth3,Cth3,-Cph3,Sph3).bstZ(-Beta34).rot(Sth5,Cth5,Sph5,Cph5).bstZ(beta12);
    *P[4] = Vec4(0.,0.,-Pf3,Enf4,Mf4).rot(Sth3,Cth3,-Cph3,Sph3).bstZ(-Beta34).rot(Sth5,Cth5,Sph5,Cph5).bstZ(beta12);
    *P[5] = Vec4(0.,0.,Egamma,Egamma,0.).rot(Sth5,Cth5,Sph5,Cph5).bstZ(beta12);
    setMomenta(proc.event);
//--------------------------------------------------------------------------------------------------
    Cth13 = costheta(*P[1],*P[3]);
    Cth14 = costheta(*P[1],*P[4]);
    Theta13 = 180./consts_.pi*acos(Cth13);
    Theta14 = 180./consts_.pi*acos(Cth14);
    Pt3 = P[3]->pT();
    Pt4 = P[4]->pT();
//--------------------------------------------------------------------------------------------------
    if (Cth13 < info.CutMinCth) return HardXS;
    if (Cth13 > info.CutMaxCth) return HardXS;
    if (Cth14 < info.CutMinCth) return HardXS;
    if (Cth14 > info.CutMaxCth) return HardXS;
    if (Theta13 < info.CutMinTheta) return HardXS;
    if (Theta13 > info.CutMaxTheta) return HardXS;
    if (Theta14 < info.CutMinTheta) return HardXS;
    if (Theta14 > info.CutMaxTheta) return HardXS;
    if (Pt3 < info.CutMinPt) return HardXS;
    if (Pt3 > info.CutMaxPt) return HardXS;
    if (Pt4 < info.CutMinPt) return HardXS;
    if (Pt4 > info.CutMaxPt) return HardXS;
    if (Egamma < scales_.omega) return HardXS;
    if (Egamma > info.CutMaxPhotonE) return HardXS;
//--------------------------------------------------------------------------------------------------
    proc.hard(&S,&Spr,&Phi3,&Eta3,&Eta5,&Hard);
    if (Hard != Hard) return HardXS;
//--------------------------------------------------------------------------------------------------
    HardXS = Hard*Factor*Jac;
    if (HardXS < 0) proc.event.weightProc = -1.;
    HardXS = abs(HardXS);
//--------------------------------------------------------------------------------------------------
    return HardXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand::LL11(int nDim, double *Xarg) {
    int    ktype = 2;
    double LL11XS = 0.;
    double Xl[3],Xu[3],Z[3];
    double Factor = 1.;
    double Cth3,Phi3;
    double X1,X2;
    double Jac;
    double Sth3,Cph3,Sph3;
    double Betai,Betaf;
    double Beta;
    double Pf3,Enf3,Enf4;
    double S,S_2,T,U,Qs,Qt,Qu;
    double n_sm,n_dm;
    double Cth13,Cth14,Pt3,Pt4,M34;
    double Born,Soft,Hard,Sigma;
    double FactorLL,S0;
    Vec4   P[6];
    proc.event.weightProc = 1.;
//--------------------------------------------------------------------------------------------------
    Xl[0] = -1.;
    Xu[0] = 1.;
    Xl[1] = 0.;
    Xu[1] = 2.*consts_.pi;
    Xl[2] = 0.;
    Xu[2] = 1.-info.DeltaLL;
    for (int i = 0; i < 3; i++) {
	Z[i] = Xl[i]+(Xu[i]-Xl[i])*Xarg[i];
	Factor *= Xu[i]-Xl[i];
    }
//--------------------------------------------------------------------------------------------------
    Cth3 = Z[0];
    Phi3 = Z[1];
    X1 = Z[2];
    X2 = 1.;
//--------------------------------------------------------------------------------------------------
    Sth3 = sqrt(1.-Cth3*Cth3);
    Cph3 = cos(Phi3);
    Sph3 = sin(Phi3);
    S0 = pow2(info.Ecm);
    S = pow2(info.Ecm)*X1;
    if (S < (Mf3+Mf4)*(Mf3+Mf4)) return LL11XS;
    S_2 = pow2(S);
    Betai = sqrt((S-pow2(Mf1+Mf2))*(S-pow2(Mf1-Mf2)))/S;
    Betaf = sqrt((S-pow2(Mf3+Mf4))*(S-pow2(Mf3-Mf4)))/S;
    Beta  = (X1-1.)/(X1+1.);
    Mf1_2 = pow2(Mf1);
    Mf2_2 = pow2(Mf2);
    Mf3_2 = pow2(Mf3);
    Mf4_2 = pow2(Mf4);
    Pf3 = sqrt((S-pow2(Mf3+Mf4))*(S-pow2(Mf3-Mf4)))/2./info.Ecm;
    Enf3 = (S+Mf3_2-Mf4_2)/2./info.Ecm;
    Enf4 = (S+Mf4_2-Mf3_2)/2./info.Ecm;
    n_sm = sm/S;
    n_dm = dm/S_2;
    T = -0.5*S*(1.+n_dm-n_sm+Betai*Betaf*Cth3);
    U = -0.5*S*(1.-n_dm-n_sm-Betai*Betaf*Cth3);
    Jac = 1./2./consts_.pi;
//--------------------------------------------------------------------------------------------------
    P[0] = Vec4();
    P[1] = Vec4(0.,0.,Betai*info.Ecm/2.,info.Ecm/2.,Mf1);
    P[2] = Vec4(0.,0.,-Betai*info.Ecm/2.,info.Ecm/2.,Mf2);
    P[3] = Vec4(0.,0.,Pf3,Enf3,Mf3).rot(Sth3,Cth3,Sph3,Cph3).bstZ(Beta);
    P[4] = Vec4(0.,0.,-Pf3,Enf4,Mf4).rot(Sth3,Cth3,Sph3,Cph3).bstZ(Beta);
    P[5] = Vec4();
//--------------------------------------------------------------------------------------------------
    Cth13 = costheta(P[1],P[3]);
    Cth14 = costheta(P[1],P[4]);
    Pt3 = P[3].pT();
    Pt4 = P[4].pT();
    M34 = m(P[3],P[4]);
//--------------------------------------------------------------------------------------------------
    if (Cth13 < info.CutMinCth) return LL11XS;
    if (Cth13 > info.CutMaxCth) return LL11XS;
    if (Cth14 < info.CutMinCth) return LL11XS;
    if (Cth14 > info.CutMaxCth) return LL11XS;
    if (Pt3 < info.CutMinPt) return LL11XS;
    if (Pt3 > info.CutMaxPt) return LL11XS;
    if (Pt4 < info.CutMinPt) return LL11XS;
    if (Pt4 > info.CutMaxPt) return LL11XS;
    if (M34 < info.CutMinM34) return LL11XS;
    if (M34 > info.CutMaxM34) return LL11XS;
//--------------------------------------------------------------------------------------------------
    isrlla_factor_(&ktype,&S0,&X1,&X2,&FactorLL);
//--------------------------------------------------------------------------------------------------
    Qs = -S;
    Qt = -T;
    Qu = -U;
    flags_.iborn = 0;
//--------------------------------------------------------------------------------------------------
    proc.born(&Qs,&Qt,&Qu,&Born,&Soft,&Hard);
//--------------------------------------------------------------------------------------------------
    LL11XS = Born*Factor*Jac*FactorLL;
    if (LL11XS < 0) proc.event.weightProc = -1.;
    LL11XS = abs(LL11XS);
//--------------------------------------------------------------------------------------------------
    proc.event.P(0)=P[0];
    proc.event.P(1)=P[1];
    proc.event.P(2)=P[2];
    proc.event.P(3)=P[3];
    proc.event.P(4)=P[4];
    proc.event.P(5)=P[5];
//--------------------------------------------------------------------------------------------------
    return LL11XS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand::LL22(int nDim, double *Xarg) {
    int    ktype = 3;
    double LL22XS = 0.;
    double Xl[3],Xu[3],Z[3];
    double Factor = 1.;
    double Cth3,Phi3;
    double X1,X2;
    double Jac;
    double Sth3,Cph3,Sph3;
    double Betai,Betaf;
    double Beta;
    double Pf3,Enf3,Enf4;
    double S,S_2,T,U,Qs,Qt,Qu;
    double n_sm,n_dm;
    double Cth13,Cth14,Pt3,Pt4,M34;
    double Born,Soft,Hard,Sigma;
    double FactorLL,S0;
    Vec4   P[6];
    proc.event.weightProc = 1.;
//--------------------------------------------------------------------------------------------------
    Xl[0] = -1.;
    Xu[0] = 1.;
    Xl[1] = 0.;
    Xu[1] = 2.*consts_.pi;
    Xl[2] = 0.;
    Xu[2] = 1.-info.DeltaLL;
    for (int i = 0; i < 3; i++) {
	Z[i] = Xl[i]+(Xu[i]-Xl[i])*Xarg[i];
	Factor *= Xu[i]-Xl[i];
    }
//--------------------------------------------------------------------------------------------------
    Cth3 = Z[0];
    Phi3 = Z[1];
    X1 = 1.;
    X2 = Z[2];
//--------------------------------------------------------------------------------------------------
    Sth3 = sqrt(1.-Cth3*Cth3);
    Cph3 = cos(Phi3);
    Sph3 = sin(Phi3);
    S0 = pow2(info.Ecm);
    S = pow2(info.Ecm)*X2;
    if (S < (Mf3+Mf4)*(Mf3+Mf4)) return LL22XS;
    S_2 = pow2(S);
    Betai = sqrt((S-pow2(Mf1+Mf2))*(S-pow2(Mf1-Mf2)))/S;
    Betaf = sqrt((S-pow2(Mf3+Mf4))*(S-pow2(Mf3-Mf4)))/S;
    Beta  = (1.-X2)/(1.+X2);
    Mf1_2 = pow2(Mf1);
    Mf2_2 = pow2(Mf2);
    Mf3_2 = pow2(Mf3);
    Mf4_2 = pow2(Mf4);
    Pf3 = sqrt((S-pow2(Mf3+Mf4))*(S-pow2(Mf3-Mf4)))/2./info.Ecm;
    Enf3 = (S+Mf3_2-Mf4_2)/2./info.Ecm;
    Enf4 = (S+Mf4_2-Mf3_2)/2./info.Ecm;
    n_sm = sm/S;
    n_dm = dm/S_2;
    T = -0.5*S*(1.+n_dm-n_sm+Betai*Betaf*Cth3);
    U = -0.5*S*(1.-n_dm-n_sm-Betai*Betaf*Cth3);
    Jac = 1./2./consts_.pi;
//--------------------------------------------------------------------------------------------------
    P[0] = Vec4();
    P[1] = Vec4(0.,0.,Betai*info.Ecm/2.,info.Ecm/2.,Mf1);
    P[2] = Vec4(0.,0.,-Betai*info.Ecm/2.,info.Ecm/2.,Mf2);
    P[3] = Vec4(0.,0.,Pf3,Enf3,Mf3).rot(Sth3,Cth3,Sph3,Cph3).bstZ(Beta);
    P[4] = Vec4(0.,0.,-Pf3,Enf4,Mf4).rot(Sth3,Cth3,Sph3,Cph3).bstZ(Beta);
    P[5] = Vec4();
//--------------------------------------------------------------------------------------------------
    Cth13 = costheta(P[1],P[3]);
    Cth14 = costheta(P[1],P[4]);
    Pt3 = P[3].pT();
    Pt4 = P[4].pT();
    M34 = m(P[3],P[4]);
//--------------------------------------------------------------------------------------------------
    if (Cth13 < info.CutMinCth) return LL22XS;
    if (Cth13 > info.CutMaxCth) return LL22XS;
    if (Cth14 < info.CutMinCth) return LL22XS;
    if (Cth14 > info.CutMaxCth) return LL22XS;
    if (Pt3 < info.CutMinPt) return LL22XS;
    if (Pt3 > info.CutMaxPt) return LL22XS;
    if (Pt4 < info.CutMinPt) return LL22XS;
    if (Pt4 > info.CutMaxPt) return LL22XS;
    if (M34 < info.CutMinM34) return LL22XS;
    if (M34 > info.CutMaxM34) return LL22XS;
//--------------------------------------------------------------------------------------------------
    isrlla_factor_(&ktype,&S0,&X1,&X2,&FactorLL);
//--------------------------------------------------------------------------------------------------
    Qs = -S;
    Qt = -T;
    Qu = -U;
    flags_.iborn = 0;
//--------------------------------------------------------------------------------------------------
    proc.born(&Qs,&Qt,&Qu,&Born,&Soft,&Hard);
//--------------------------------------------------------------------------------------------------
    LL22XS = Born*Factor*Jac*FactorLL;
    if (LL22XS < 0) proc.event.weightProc = -1.;
    LL22XS = abs(LL22XS);
//--------------------------------------------------------------------------------------------------
    proc.event.P(0)=P[0];
    proc.event.P(1)=P[1];
    proc.event.P(2)=P[2];
    proc.event.P(3)=P[3];
    proc.event.P(4)=P[4];
    proc.event.P(5)=P[5];
//--------------------------------------------------------------------------------------------------
    return LL22XS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand::LL12(int nDim, double *Xarg) {
    int    ktype = 4;
    double LL12XS = 0.;
    double Xl[4],Xu[4],Z[4];
    double Factor = 1.;
    double Cth3,Phi3;
    double X1,X2;
    double Jac;
    double Sth3,Cph3,Sph3;
    double Betai,Betaf;
    double Beta;
    double Pf3,Enf3,Enf4;
    double S,S_2,T,U,Qs,Qt,Qu;
    double n_sm,n_dm;
    double Cth13,Cth14,Pt3,Pt4,M34;
    double Born,Soft,Hard,Sigma;
    double FactorLL,S0;
    Vec4   P[6];
    proc.event.weightProc = 1.;
//--------------------------------------------------------------------------------------------------
    Xl[0] = -1.;
    Xu[0] = 1.;
    Xl[1] = 0.;
    Xu[1] = 2.*consts_.pi;
    Xl[2] = 0.;
    Xu[2] = 1.-info.DeltaLL;
    Xl[3] = 0.;
    Xu[3] = 1.-info.DeltaLL;
    for (int i = 0; i < 4; i++) {
	Z[i] = Xl[i]+(Xu[i]-Xl[i])*Xarg[i];
	Factor *= Xu[i]-Xl[i];
    }
//--------------------------------------------------------------------------------------------------
    Cth3 = Z[0];
    Phi3 = Z[1];
    X1 = Z[2];
    X2 = Z[3];
//--------------------------------------------------------------------------------------------------
    Sth3 = sqrt(1.-Cth3*Cth3);
    Cph3 = cos(Phi3);
    Sph3 = sin(Phi3);
    S0 = pow2(info.Ecm);
    S = pow2(info.Ecm)*X1*X2;
    if (S < (Mf3+Mf4)*(Mf3+Mf4)) return LL12XS;
    S_2 = pow2(S);
    Betai = sqrt((S-pow2(Mf1+Mf2))*(S-pow2(Mf1-Mf2)))/S;
    Betaf = sqrt((S-pow2(Mf3+Mf4))*(S-pow2(Mf3-Mf4)))/S;
    Beta  = (X1-X2)/(X1+X2);
    Mf1_2 = pow2(Mf1);
    Mf2_2 = pow2(Mf2);
    Mf3_2 = pow2(Mf3);
    Mf4_2 = pow2(Mf4);
    Pf3 = sqrt((S-pow2(Mf3+Mf4))*(S-pow2(Mf3-Mf4)))/2./info.Ecm;
    Enf3 = (S+Mf3_2-Mf4_2)/2./info.Ecm;
    Enf4 = (S+Mf4_2-Mf3_2)/2./info.Ecm;
    n_sm = sm/S;
    n_dm = dm/S_2;
    T = -0.5*S*(1.+n_dm-n_sm+Betai*Betaf*Cth3);
    U = -0.5*S*(1.-n_dm-n_sm-Betai*Betaf*Cth3);
    Jac = 1./2./consts_.pi;
//--------------------------------------------------------------------------------------------------
    P[0] = Vec4();
    P[1] = Vec4(0.,0.,Betai*info.Ecm/2.,info.Ecm/2.,Mf1);
    P[2] = Vec4(0.,0.,-Betai*info.Ecm/2.,info.Ecm/2.,Mf2);
    P[3] = Vec4(0.,0.,Pf3,Enf3,Mf3).rot(Sth3,Cth3,Sph3,Cph3).bstZ(Beta);
    P[4] = Vec4(0.,0.,-Pf3,Enf4,Mf4).rot(Sth3,Cth3,Sph3,Cph3).bstZ(Beta);
    P[5] = Vec4();
//--------------------------------------------------------------------------------------------------
    Cth13 = costheta(P[1],P[3]);
    Cth14 = costheta(P[1],P[4]);
    Pt3 = P[3].pT();
    Pt4 = P[4].pT();
    M34 = m(P[3],P[4]);
//--------------------------------------------------------------------------------------------------
    if (Cth13 < info.CutMinCth) return LL12XS;
    if (Cth13 > info.CutMaxCth) return LL12XS;
    if (Cth14 < info.CutMinCth) return LL12XS;
    if (Cth14 > info.CutMaxCth) return LL12XS;
    if (Pt3 < info.CutMinPt) return LL12XS;
    if (Pt3 > info.CutMaxPt) return LL12XS;
    if (Pt4 < info.CutMinPt) return LL12XS;
    if (Pt4 > info.CutMaxPt) return LL12XS;
    if (M34 < info.CutMinM34) return LL12XS;
    if (M34 > info.CutMaxM34) return LL12XS;
//--------------------------------------------------------------------------------------------------
    isrlla_factor_(&ktype,&S0,&X1,&X2,&FactorLL);
//--------------------------------------------------------------------------------------------------
    Qs = -S;
    Qt = -T;
    Qu = -U;
    flags_.iborn = 0;
//--------------------------------------------------------------------------------------------------
    proc.born(&Qs,&Qt,&Qu,&Born,&Soft,&Hard);
//--------------------------------------------------------------------------------------------------
    LL12XS = Born*Factor*Jac*FactorLL;
    if (LL12XS < 0) proc.event.weightProc = -1.;
    LL12XS = abs(LL12XS);
    if (LL12XS != LL12XS) LL12XS = 0.;
//--------------------------------------------------------------------------------------------------
    proc.event.P(0)=P[0];
    proc.event.P(1)=P[1];
    proc.event.P(2)=P[2];
    proc.event.P(3)=P[3];
    proc.event.P(4)=P[4];
    proc.event.P(5)=P[5];
//--------------------------------------------------------------------------------------------------
    return LL12XS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand::Loop(int nDim, double *Xarg){
    double LoopXS(0.);
    double Xl[5],Xu[5],Z[6];
    double Factor = 1.;
    double FactorBorn = 1.;
    double Spr,Eta3,Phi3,Eta5,Phi5;
    double Jac;
    double Cth3,Sth3,Cph3,Sph3,Cth5,Sth5,Cph5,Sph5;
    double Beta34,beta12;
    double Pf3,Enf3,Enf4;
    double S,Sqspr,Qs,Qt,Qu,Tn,Un;
    double Cth13,Cth14,Theta13,Theta14,Pt3,Pt4;
    double Egamma;
    double Born(0.),Soft(0.),Hard(0.),Sigma(0.);
    Vec4   P[6];
    proc.event.weightProc = 1.;
    if ((info.Ecm<Mf1+Mf2)||(info.Ecm<Mf3+Mf4)) return 0.;
//--------------------------------------------------------------------------------------------------
    Z[5] = Xarg[5];
    if (Z[5] < 0.5) {
	Xl[1] = -1.0;
	Xu[1] = 1.0;
	Xl[4] = 0.0;
	Xu[4] = 2.0*consts_.pi;
    } else {
	Xl[0] = log(info.Ome);
	Xu[0] = log(1.-(Mf3+Mf4)*(Mf3+Mf4)/(info.Ecm*info.Ecm));
	Xl[1] = -100.0;
	Xu[1] = 100.0;
	Xl[2] = 0.0;
	Xu[2] = 2.0*consts_.pi;
	Xl[3] = -100.0;
	Xu[3] = 100.0;
	Xl[4] = 0.0;
	Xu[4] = 2.0*consts_.pi;
    }
    for (int i = 0; i < 5; i++) {
	Z[i] = Xl[i]+(Xu[i]-Xl[i])*Xarg[i];
	Factor *= Xu[i]-Xl[i];
	if (i == 1 || i == 4) FactorBorn *= Xu[i]-Xl[i];
    }
//--------------------------------------------------------------------------------------------------
    S = info.Ecm*info.Ecm;
    Spr = S*(1.-exp(Z[0]));
    Eta3 = Z[1];
    Phi3 = Z[2];
    Eta5 = Z[3];
    Phi5 = Z[4];
//--------------------------------------------------------------------------------------------------
    if (Z[5] < 0.5) {
	Cth3 = Z[1];
	Sth3 = sqrt(1.-Cth3*Cth3);
	Spr = S;
	Sth5 = 0.;
	Cth5 = 1.;
	Sph3 = 1.;
	Cph3 = 0.;
	Egamma = 0.;
    } else {
	Cth3 = (exp(2.0*Eta3)-1.0)/(exp(2.0*Eta3)+1.0);
	Sth3 = 2.0*exp(Eta3)/(exp(2.0*Eta3)+1.0);
	Cph3 = cos(Phi3);
	Sph3 = sin(Phi3);
	Cth5 = (exp(2.0*Eta5)-1.0)/(exp(2.0*Eta5)+1.0);
	Sth5 = 2.0*exp(Eta5)/(exp(2.0*Eta5)+1.0);
	Egamma = info.Ecm*exp(Z[0])/2.0;
    }
    Cph5 = cos(Phi5);
    Sph5 = sin(Phi5);
    beta12 = (proc.beta1*proc.E1-proc.beta2*proc.E2)/(proc.E1+proc.E2);
    Sqspr = sqrt(Spr);
    Mf1_2 = pow2(Mf1);
    Mf2_2 = pow2(Mf2);
    Mf3_2 = pow2(Mf3);
    Mf4_2 = pow2(Mf4);
    Pf3 = sqrt((Spr-pow2(Mf3+Mf4))*(Spr-pow2(Mf3-Mf4)))/2./Sqspr;
    Enf3 = (Spr+Mf3_2-Mf4_2)/2./Sqspr;
    Enf4 = (Spr+Mf4_2-Mf3_2)/2./Sqspr;
    if (Z[5] < 0.5) {
	Jac = 1./2./consts_.pi;
	Beta34 = 0.;
    }
    else {
	Jac = Sth3*Sth3*Sth5*Sth5/2./consts_.pi*S*exp(Z[0]);
	Beta34 = exp(Z[0])/(2.-exp(Z[0]));
    }
//--------------------------------------------------------------------------------------------------
// NOTE: Because SANC use special reference frame,
// such as phi3=phi3'+pi/2 and th5 is photon`s angle,
// we need replace Sph3->-Cph3, Cph3->Sph3 and boost in opposite direction
    P[0] = Vec4();
    P[1] = Vec4(0.,0.,proc.beta1*proc.E1,proc.E1,Mf1);
    P[2] = Vec4(0.,0.,-proc.beta2*proc.E2,proc.E2,Mf2);
    P[3] = Vec4(0.,0.,Pf3,Enf3,Mf3).rot(Sth3,Cth3,-Cph3,Sph3).bstZ(-Beta34).rot(Sth5,Cth5,Sph5,Cph5).bstZ(beta12);
    P[4] = Vec4(0.,0.,-Pf3,Enf4,Mf4).rot(Sth3,Cth3,-Cph3,Sph3).bstZ(-Beta34).rot(Sth5,Cth5,Sph5,Cph5).bstZ(beta12);
    P[5] = Vec4(0.,0.,Egamma,Egamma,0.).rot(Sth5,Cth5,Sph5,Cph5).bstZ(beta12);
//--------------------------------------------------------------------------------------------------
    Cth13 = costheta(P[1],P[3]);
    Cth14 = costheta(P[1],P[4]);
    Theta13 = 180./consts_.pi*acos(Cth13);
    Theta14 = 180./consts_.pi*acos(Cth14);
    Pt3 = P[3].pT();
    Pt4 = P[4].pT();
    Tn = t(P[1],P[4]);
    Un = t(P[1],P[3]);
//--------------------------------------------------------------------------------------------------
    if (Cth13 < info.CutMinCth) return LoopXS;
    if (Cth13 > info.CutMaxCth) return LoopXS;
    if (Cth14 < info.CutMinCth) return LoopXS;
    if (Cth14 > info.CutMaxCth) return LoopXS;
    if (Theta13 < info.CutMinTheta) return LoopXS;
    if (Theta13 > info.CutMaxTheta) return LoopXS;
    if (Theta14 < info.CutMinTheta) return LoopXS;
    if (Theta14 > info.CutMaxTheta) return LoopXS;
    if (Pt3 < info.CutMinPt) return LoopXS;
    if (Pt3 > info.CutMaxPt) return LoopXS;
    if (Pt4 < info.CutMinPt) return LoopXS;
    if (Pt4 > info.CutMaxPt) return LoopXS;
    if (Z[5] >= 0.5) {
	if (Egamma < scales_.omega) return LoopXS;
    if (Egamma > info.CutMaxPhotonE) return LoopXS;
    }
//--------------------------------------------------------------------------------------------------
    Qs = -S;
    Qt = -Tn;
    Qu = -Un;
    flags_.iborn = 0;
//--------------------------------------------------------------------------------------------------
    if (Z[5] >= 0.5) {
        proc.hard(&S,&Spr,&Phi3,&Eta3,&Eta5,&Hard);
    } else {
        proc.born(&Qs,&Qt,&Qu,&Born,&Soft,&Hard);
        proc.virt(&S,&Tn,&Un,&Sigma);
    }
//--------------------------------------------------------------------------------------------------
    if (Z[5] >= 0.5) {
	LoopXS = 2.*Hard*Factor*Jac;
    } else {
	LoopXS = 2.*(Sigma+Soft)*FactorBorn*Jac;
    }
    if (LoopXS < 0) proc.event.weightProc = -1.;
    LoopXS = abs(LoopXS);
    if (LoopXS!=LoopXS) {
	cout<<"Error:"<<Z[0]<<" "<<Z[1]<<" "<<Z[2]<<" "<<Z[3]<<" "<<Z[4]<<" "<<Z[5]<<endl;
	cout<<Sigma<<" "<<Soft<<" "<<Hard<<" "<<endl;
	cout<<S<<" "<<Spr<<" "<<Phi3<<" "<<Eta3<<" "<<Eta5<<endl;
    }
//--------------------------------------------------------------------------------------------------
    proc.event.P(0)=P[0];
    proc.event.P(1)=P[1];
    proc.event.P(2)=P[2];
    proc.event.P(3)=P[3];
    proc.event.P(4)=P[4];
    proc.event.P(5)=P[5];
//--------------------------------------------------------------------------------------------------
    return LoopXS;
}
