#include "ReneSANCe_ee.h"
#include "TRandom3.h"
#include <iostream>
#include <iomanip>
#include <sys/stat.h>
#include <complex>
using namespace std;
typedef complex<double> double_complex;
#include "clooptools.h"
#include "TFoam.h"
#include "Integrand.h"
#include "TFile.h"
#include "TTree.h"
#include "Info.h"
#include "Process.h"
#include "ModulesInterface.h"
#include "Time.h"
#include "DataSaver.h"


extern void LHEInit(void);

using namespace SANC;

#define SP12 setprecision(12) << setw(21)
#define SW18 setw(18)
#define SW3  setw(3)
#define SW6  setw(6)


ReneSANCe_ee::ReneSANCe_ee(Settings & _cfg, Info & _info, Process & _proc, ModulesInterface & _mod, Time & _time, DataSaver & _ds):
cfg(_cfg),timing(_time),info(_info),proc(_proc),modules(_mod),ds(_ds)
{
    PseRan=new TRandom3();
    PseRan->SetSeed(info.RanSeed);
    ffini();
}

void ReneSANCe_ee::generateGrid()
{

}

void ReneSANCe_ee::generateEvents()
{
    TFoam *FoamBorn,*FoamVirt,*FoamHard,*FoamLL11,*FoamLL22,*FoamLL12;
    TFoamIntegrand *Density0,*Density1,*Density2,*Density4,*Density5,*Density6;
    TFile *fileBorn,*fileVirt,*fileHard,*fileLL11,*fileLL22,*fileLL12;
    string bornName, bornNameLhe, bornNameRoot, bornNameDat;
    string loopName, loopNameLhe, loopNameRoot, loopNameDat;
    bool bBorn,bVirt,bHard;
    bBorn = true;
    bVirt = !info.Iborn && !info.ForceOffSV && (info.Iew || info.Iqed || info.Iqedll);
    bHard = !info.Iborn && !info.ForceOffH && info.Iqed;
    bornName = "SANCeeLO";
    loopName = "SANCeeNLO";
    if (info.Alr == 1) {
		bornName = bornName+"dif";
		loopName = loopName+"dif";
    } else if (info.Alr == 2) {
		bornName = bornName + "sum";
		loopName = loopName + "sum";
    }
    bornNameLhe = info.outputDir+"/"+bornName+".lhe";
    bornNameRoot = info.outputDir+"/"+bornName+".root";
    bornNameDat = info.outputDir+"/"+bornName+".dat";
    loopNameLhe = info.outputDir+"/"+loopName+".lhe";
    loopNameRoot = info.outputDir+"/"+loopName+".root";
    loopNameDat = info.outputDir+"/"+loopName+".dat";
    timing.startClock();
    if (info.IEXPLBORN) {
	FoamBorn = new TFoam("FoamBorn");   // Create Simulator
	Density0 = new Integrand(info, proc, 0);   // Create integrand function
	FoamBorn->SetkDim(proc.kDimBorn);
	FoamBorn->SetnCells(info.nCellsBorn);
	FoamBorn->SetnSampl(info.nSamplBorn);
	FoamBorn->SetnBin(info.nBinBorn);
	FoamBorn->SetEvPerBin(info.EvPerBinBorn);
	FoamBorn->SetOptRej(info.OptRej);
	FoamBorn->SetOptDrive(info.OptDrive);
	FoamBorn->SetMaxWtRej(info.MaxWtRejBorn);
	FoamBorn->SetChat(info.Chat);
	FoamBorn->SetRho(Density0);
	FoamBorn->SetPseRan(PseRan);
	FoamBorn->SetInhiDiv(1, 1);
	FoamBorn->Initialize();
	TFile RootBornFile("FoamBorn.root","RECREATE","Born");
	FoamBorn->Write("FoamBorn");    // Writing Foam on the disk
    } else {
	fileBorn = new TFile("FoamBorn.root"); // connect disk file
	FoamBorn = (TFoam*)fileBorn->Get("FoamBorn"); // find object
	Density0 = new Integrand(info, proc, 0);   // Create integrand function
	FoamBorn->SetPseRan(PseRan);
	FoamBorn->SetRho(Density0);
	FoamBorn->SetMaxWtRej(info.MaxWtRejBorn);
	FoamBorn->LinkCells(); // restore pointers of the binary tree of cells
	FoamBorn->GetTotDim();
    }
    timing.stopClock();
    timing.ReportClock("Born initialization");
	FoamBorn->GetPrimary(MCresult);
	if (MCresult<1.e-300) {
        bBorn = false;
        bVirt = false;
    }
    if (bVirt) {
	timing.startClock();
	if (info.IEXPLVIRT) {
	    FoamVirt = new TFoam("FoamVirt");   // Create Simulator
	    Density1 = new Integrand(info, proc, 1);   // Create integrand function
	    FoamVirt->SetkDim(proc.kDimVirt);
	    FoamVirt->SetnCells(info.nCellsVirt);
	    FoamVirt->SetnSampl(info.nSamplVirt);
	    FoamVirt->SetnBin(info.nBinVirt);
	    FoamVirt->SetEvPerBin(info.EvPerBinVirt);
	    FoamVirt->SetOptRej(info.OptRej);
	    FoamVirt->SetOptDrive(info.OptDrive);
	    FoamVirt->SetMaxWtRej(info.MaxWtRejVirt);
	    FoamVirt->SetChat(info.Chat);
	    FoamVirt->SetRho(Density1);
	    FoamVirt->SetPseRan(PseRan);
	    FoamVirt->SetInhiDiv(1, 1);
	    FoamVirt->Initialize();
	    TFile RootVirtFile("FoamVirt.root","RECREATE","Virt");
	    FoamVirt->Write("FoamVirt");    // Writing Foam on the disk
	} else {
	    fileVirt = new TFile("FoamVirt.root"); // connect disk file
	    FoamVirt = (TFoam*)fileVirt->Get("FoamVirt"); // find object
	    Density1 = new Integrand(info, proc, 1);   // Create integrand function
	    FoamVirt->SetPseRan(PseRan);
	    FoamVirt->SetRho(Density1);
	    FoamVirt->SetMaxWtRej(info.MaxWtRejVirt);
	    FoamVirt->LinkCells(); // restore pointers of the binary tree of cells
	    FoamVirt->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("virt initialization");
    }
    if (bHard) {
	timing.startClock();
	if (info.IEXPLHARD) {
	    FoamHard = new TFoam("FoamHard");   // Create Simulator
	    Density2 = new Integrand(info, proc, 2);   // Create integrand function
	    FoamHard->SetkDim(proc.kDimHard);
	    FoamHard->SetnCells(info.nCellsHard);
	    FoamHard->SetnSampl(info.nSamplHard);
	    FoamHard->SetnBin(info.nBinHard);
	    FoamHard->SetEvPerBin(info.EvPerBinHard);
	    FoamHard->SetOptRej(info.OptRej);
	    FoamHard->SetOptDrive(info.OptDrive);
	    FoamHard->SetMaxWtRej(info.MaxWtRejHard);
	    FoamHard->SetChat(info.Chat);
	    FoamHard->SetRho(Density2);
	    FoamHard->SetPseRan(PseRan);
	    FoamHard->SetInhiDiv(4, 1);
	    FoamHard->Initialize();
	    TFile RootHardFile("FoamHard.root","RECREATE","Hard");
	    FoamHard->Write("FoamHard");    // Writing Foam on the disk
	} else {
	    fileHard = new TFile("FoamHard.root"); // connect disk file
	    FoamHard = (TFoam*)fileHard->Get("FoamHard"); // find object
	    Density2 = new Integrand(info, proc, 2);   // Create integrand function
	    FoamHard->SetPseRan(PseRan);
	    FoamHard->SetRho(Density2);
	    FoamHard->SetMaxWtRej(info.MaxWtRejHard);
	    FoamHard->LinkCells(); // restore pointers of the binary tree of cells
	    FoamHard->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("hard initialization");
    FoamHard->GetPrimary(MCresult);
	if (MCresult<1.e-300) { bHard = false; }
    }
    if (!info.Iborn && info.Iqedll) {
	timing.startClock();
	if (info.IEXPLLL11) {
	    FoamLL11 = new TFoam("FoamLL11");   // Create Simulator
	    Density4 = new Integrand(info, proc, 4);   // Create integrand function
	    FoamLL11->SetkDim(proc.kDimLL11);
	    FoamLL11->SetnCells(info.nCellsLL11);
	    FoamLL11->SetnSampl(info.nSamplLL11);
	    FoamLL11->SetnBin(info.nBinLL11);
	    FoamLL11->SetEvPerBin(info.EvPerBinLL11);
	    FoamLL11->SetOptRej(info.OptRej);
	    FoamLL11->SetOptDrive(info.OptDrive);
	    FoamLL11->SetMaxWtRej(info.MaxWtRejLL11);
	    FoamLL11->SetChat(info.Chat);
	    FoamLL11->SetRho(Density4);
	    FoamLL11->SetPseRan(PseRan);
	    FoamLL11->SetInhiDiv(1, 1);
	    FoamLL11->Initialize();
	    TFile RootLL11File("FoamLL11.root","RECREATE","LL11");
	    FoamLL11->Write("FoamLL11");    // Writing Foam on the disk
	} else {
	    fileLL11 = new TFile("FoamLL11.root"); // connect disk file
	    FoamLL11 = (TFoam*)fileLL11->Get("FoamLL11"); // find object
	    Density4 = new Integrand(info, proc, 4);   // Create integrand function
	    FoamLL11->SetPseRan(PseRan);
	    FoamLL11->SetRho(Density4);
	    FoamLL11->SetMaxWtRej(info.MaxWtRejLL11);
	    FoamLL11->LinkCells(); // restore pointers of the binary tree of cells
	    FoamLL11->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("ll11 initialization");
    }
    if (!info.Iborn && info.Iqedll) {
	timing.startClock();
	if (info.IEXPLLL22) {
	    FoamLL22 = new TFoam("FoamLL22");   // Create Simulator
	    Density5 = new Integrand(info, proc, 5);   // Create integrand function
	    FoamLL22->SetkDim(proc.kDimLL22);
	    FoamLL22->SetnCells(info.nCellsLL22);
	    FoamLL22->SetnSampl(info.nSamplLL22);
	    FoamLL22->SetnBin(info.nBinLL22);
	    FoamLL22->SetEvPerBin(info.EvPerBinLL22);
	    FoamLL22->SetOptRej(info.OptRej);
	    FoamLL22->SetOptDrive(info.OptDrive);
	    FoamLL22->SetMaxWtRej(info.MaxWtRejLL22);
	    FoamLL22->SetChat(info.Chat);
	    FoamLL22->SetRho(Density5);
	    FoamLL22->SetPseRan(PseRan);
	    FoamLL22->SetInhiDiv(1, 1);
	    FoamLL22->Initialize();
	    TFile RootLL22File("FoamLL22.root","RECREATE","LL22");
	    FoamLL22->Write("FoamLL22");    // Writing Foam on the disk
	} else {
	    fileLL22 = new TFile("FoamLL22.root"); // connect disk file
	    FoamLL22 = (TFoam*)fileLL22->Get("FoamLL22"); // find object
	    Density5 = new Integrand(info, proc, 5);   // Create integrand function
	    FoamLL22->SetPseRan(PseRan);
	    FoamLL22->SetRho(Density5);
	    FoamLL22->SetMaxWtRej(info.MaxWtRejLL22);
	    FoamLL22->LinkCells(); // restore pointers of the binary tree of cells
	    FoamLL22->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("ll22 initialization");
    }
    if (!info.Iborn && (info.Iqedll >1) && (info.Iqedll !=3) && (info.Iqedll !=4)) {
	timing.startClock();
	if (info.IEXPLLL12) {
	    FoamLL12 = new TFoam("FoamLL12");   // Create Simulator
	    Density6 = new Integrand(info, proc, 6);   // Create integrand function
	    FoamLL12->SetkDim(proc.kDimLL12);
	    FoamLL12->SetnCells(info.nCellsLL12);
	    FoamLL12->SetnSampl(info.nSamplLL12);
	    FoamLL12->SetnBin(info.nBinLL12);
	    FoamLL12->SetEvPerBin(info.EvPerBinLL12);
	    FoamLL12->SetOptRej(info.OptRej);
	    FoamLL12->SetOptDrive(info.OptDrive);
	    FoamLL12->SetMaxWtRej(info.MaxWtRejLL12);
	    FoamLL12->SetChat(info.Chat);
	    FoamLL12->SetRho(Density6);
	    FoamLL12->SetPseRan(PseRan);
//	    FoamLL12->SetXdivPRD(2, 1, X1Div);
//	    FoamLL12->SetXdivPRD(3, 1, X2Div);
	    FoamLL12->SetInhiDiv(1, 1);
	    FoamLL12->Initialize();
	    TFile RootLL12File("FoamLL12.root","RECREATE","LL12");
	    FoamLL12->Write("FoamLL12");    // Writing Foam on the disk
	} else {
	    fileLL12 = new TFile("FoamLL12.root"); // connect disk file
	    FoamLL12 = (TFoam*)fileLL12->Get("FoamLL12"); // find object
	    Density6 = new Integrand(info, proc, 6);   // Create integrand function
	    FoamLL12->SetPseRan(PseRan);
	    FoamLL12->SetRho(Density6);
	    FoamLL12->SetMaxWtRej(info.MaxWtRejLL12);
	    FoamLL12->LinkCells(); // restore pointers of the binary tree of cells
	    FoamLL12->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("ll12 initialization");
    }
    timing.startClock();
    cout << "====== Initialization done, entering MC loop" << '\n';
//--------------------------------------------------------------------------------------------------
    double *MCvectBorn = new double[proc.kDimBorn];   // vector generated in the MC run
    double *MCvectVirt = new double[proc.kDimVirt];   // vector generated in the MC run
    double *MCvectHard = new double[proc.kDimHard];   // vector generated in the MC run
    double *MCvectLL11 = new double[proc.kDimLL11];   // vector generated in the MC run
    double *MCvectLL22 = new double[proc.kDimLL22];   // vector generated in the MC run
    double *MCvectLL12 = new double[proc.kDimLL12];   // vector generated in the MC run
//--------------------------------------------------------------------------------------------------
    int iBornPWt1  = 0, iBornNWt1  = 0;
    int iBornPWtN1 = 0, iBornNWtN1 = 0;
    int iVirtPWt1  = 0, iVirtNWt1  = 0;
    int iVirtPWtN1 = 0, iVirtNWtN1 = 0;
    int iHardPWt1  = 0, iHardNWt1  = 0;
    int iHardPWtN1 = 0, iHardNWtN1 = 0;
    int iLL11PWt1  = 0, iLL11NWt1  = 0;
    int iLL11PWtN1 = 0, iLL11NWtN1 = 0;
    int iLL22PWt1  = 0, iLL22NWt1  = 0;
    int iLL22PWtN1 = 0, iLL22NWtN1 = 0;
    int iLL12PWt1  = 0, iLL12NWt1  = 0;
    int iLL12PWtN1 = 0, iLL12NWtN1 = 0;
//--------------------------------------------------------------------------------------------------
    for (int i = 0; i < 6; i++) {
        branch[i] = 0.;
        branch_err[i] = 0.;
    }
//--------------------------------------------------------------------------------------------------
    mkdir(info.outputDir.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
//--------------------------------------------------------------------------------------------------
	if (bBorn) {
		output_brn.open(bornNameLhe);
		ds.setOutput(&output_brn);
		ds.start();
//--------------------------------------------------------------------------------------------------
		TFile *fborn = new TFile(bornNameRoot.c_str(),"recreate");
		TTree *tborn = new TTree("events_born","born events");
		Double_t weight_born[1], px_born[3], py_born[3], pz_born[3], E_born[3];
		tborn->Branch("weight_born", weight_born, "weight_born[1]/D");
		tborn->Branch("px_born", px_born, "px_born[3]/D");
		tborn->Branch("py_born", py_born, "py_born[3]/D");
		tborn->Branch("pz_born", pz_born, "pz_born[3]/D");
		tborn->Branch("E_born", E_born, "E_born[3]/D");
	//--------------------------------------------------------------------------------------------------
		for (loop = 0; loop < info.NevBorn; loop++) {
		FoamBorn->MakeEvent(); // generate Born MC event
		FoamBorn->GetMCvect(MCvectBorn);
		MCwt = FoamBorn->GetMCwt();
		xwgtup = proc.event.weight();
		if (MCwt == 1.0) {
			weight_born[0] = xwgtup;
			px_born[0] = proc.event.P(3).px();
			px_born[1] = proc.event.P(4).px();
			px_born[2] = proc.event.P(5).px();
			py_born[0] = proc.event.P(3).py();
			py_born[1] = proc.event.P(4).py();
			py_born[2] = proc.event.P(5).py();
			pz_born[0] = proc.event.P(3).pz();
			pz_born[1] = proc.event.P(4).pz();
			pz_born[2] = proc.event.P(5).pz();
			E_born[0]  = proc.event.P(3).e();
			E_born[1]  = proc.event.P(4).e();
			E_born[2]  = proc.event.P(5).e();
			if (info.PrintLHE == 1) ds.writeEvent();
			if (info.PrintROOT == 1) tborn->Fill();
			if (xwgtup > 0.0) iBornPWt1++;
			if (xwgtup < 0.0) iBornNWt1++;
		} else {
			if (xwgtup > 0.0) iBornPWtN1++;
			if (xwgtup < 0.0) iBornNWtN1++;
		}
		}
		fborn->Write();
		timing.stopClock();
		timing.ReportClock("Born generation");
    	FoamBorn->GetIntegMC(MCresult, MCerror); // get MC integral
        branch[0] = MCresult;
        branch_err[0] = MCerror;
	}
//--------------------------------------------------------------------------------------------------
    output_tot.open(loopNameLhe);
	ds.setOutput(&output_tot);
	ds.start();
//--------------------------------------------------------------------------------------------------
    if (bVirt) {
        timing.startClock();
        for (loop = 0; loop < info.NevVirt; loop++) {
            FoamVirt->MakeEvent(); // generate Virt MC event
            FoamVirt->GetMCvect(MCvectVirt);
            MCwt = FoamVirt->GetMCwt();
        }
        timing.stopClock();
        timing.ReportClock("virt generation");
        FoamVirt->GetIntegMC(MCresult, MCerror);  // get MC integral
        branch[1] = MCresult;
        branch_err[1] = MCerror;
    }
//--------------------------------------------------------------------------------------------------
    if (bHard) {
        timing.startClock();
        for (loop = 0; loop < info.NevHard; loop++) {
            FoamHard->MakeEvent();   // generate Hard MC event
            FoamHard->GetMCvect(MCvectHard);
            MCwt = FoamHard->GetMCwt();
        }
        timing.stopClock();
        timing.ReportClock("hard generation");
        FoamHard->GetIntegMC(MCresult, MCerror);  // get MC integral
        branch[2] = MCresult;
        branch_err[2] = MCerror;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && info.Iqedll) {
        timing.startClock();
        for (loop = 0; loop < info.NevLL11; loop++) {
            FoamLL11->MakeEvent();   // generate LL11 MC event
            FoamLL11->GetMCvect(MCvectLL11);
            MCwt = FoamLL11->GetMCwt();
        }
        timing.stopClock();
        timing.ReportClock("ll11 generation");
        FoamLL11->GetIntegMC(MCresult, MCerror);  // get MC integral
        branch[3] = MCresult;
        branch_err[3] = MCerror;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && info.Iqedll) {
        timing.startClock();
        for (loop = 0; loop < info.NevLL22; loop++) {
            FoamLL22->MakeEvent();   // generate LL22 MC event
            FoamLL22->GetMCvect(MCvectLL22);
            MCwt = FoamLL22->GetMCwt();
        }
        timing.stopClock();
        timing.ReportClock("ll22 generation");
        FoamLL22->GetIntegMC(MCresult, MCerror);  // get MC integral
        branch[4] = MCresult;
        branch_err[4] = MCerror;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && (info.Iqedll >1) && (info.Iqedll !=3) && (info.Iqedll !=4)) {
        timing.startClock();
        for (loop = 0; loop < info.NevLL12; loop++) {
            FoamLL12->MakeEvent();   // generate LL12 MC event
            FoamLL12->GetMCvect(MCvectLL12);
            MCwt = FoamLL12->GetMCwt();
        }
        timing.stopClock();
        timing.ReportClock("ll12 generation");
        FoamLL12->GetIntegMC(MCresult, MCerror);  // get MC integral
        branch[5] = MCresult;
        branch_err[5] = MCerror;
    }
//--------------------------------------------------------------------------------------------------
    info.SigmaBorn = branch[0];
    info.SigmaVirt = branch[1];
    info.SigmaHard = branch[2];
    info.SigmaLL11 = branch[3];
    info.SigmaLL22 = branch[4];
    info.SigmaLL12 = branch[5];
    info.SigmaTot = 0.;
    for (int i = 1; i < 6; i++) {
	    info.SigmaTot += branch[i];
    }
//--------------------------------------------------------------------------------------------------
    cout << "-------------------" << '\n';
    cout << "Branching weights:" << '\n';
    cout << "Born = " << branch[0] << " +- " << branch_err[0] << '\n';
    cout << "Virt = " << branch[1] << " +- " << branch_err[1] << '\n';
    cout << "Hard = " << branch[2] << " +- " << branch_err[2] << '\n';
    cout << "LL11 = " << branch[3] << " +- " << branch_err[3] << '\n';
    cout << "LL22 = " << branch[4] << " +- " << branch_err[4] << '\n';
    cout << "LL12 = " << branch[5] << " +- " << branch_err[5] << '\n';
    cout << "-------------------" << '\n';
//--------------------------------------------------------------------------------------------------
	if (info.SigmaTot>1.e-300) {

        double rBr;
        double Br[4];
        Br[0] = branch[1]/info.SigmaTot;
        for (int i = 1; i < 4; i++) {
            Br[i] = Br[i-1]+branch[i+1]/info.SigmaTot;
        }
//--------------------------------------------------------------------------------------------------
        TFile *ftot = new TFile(loopNameRoot.c_str(),"recreate");
        TTree *ttot = new TTree("events_tot","tot events");
        Double_t weight_tot[1], px_tot[3], py_tot[3], pz_tot[3], E_tot[3];
        ttot->Branch("weight_tot", weight_tot, "weight_tot[1]/D");
        ttot->Branch("px_tot", px_tot, "px_tot[3]/D");
        ttot->Branch("py_tot", py_tot, "py_tot[3]/D");
        ttot->Branch("pz_tot", pz_tot, "pz_tot[3]/D");
        ttot->Branch("E_tot", E_tot, "E_tot[3]/D");
//--------------------------------------------------------------------------------------------------
        TFile *fhardvar = new TFile((info.outputDir+"/hardvar.root").c_str(),"recreate");
        TTree *thardvar = new TTree("events_hardvar","tot events");
        Double_t weight_hardvar[1], hardvar[5];
        thardvar->Branch("weight_hardvar", weight_hardvar, "weight_hardvar[1]/D");
        thardvar->Branch("hardvar", hardvar, "hardvar[5]/D");
//--------------------------------------------------------------------------------------------------
        timing.startClock();
        for (loop = 0; loop < info.NevTot; loop++) {
            rBr = (double) random()/RAND_MAX;
//--------------------------------------------------------------------------------------------------
            if (rBr <= Br[0] && !info.Iborn) {
                FoamVirt->MakeEvent();   // generate Virt MC event
                FoamVirt->GetMCvect(MCvectVirt);
                MCwt = FoamVirt->GetMCwt();
                xwgtup = proc.event.weight();
                if (MCwt == 1.0) {
                weight_tot[0] = xwgtup;
                px_tot[0] = proc.event.P(3).px();
                px_tot[1] = proc.event.P(4).px();
                px_tot[2] = proc.event.P(5).px();
                py_tot[0] = proc.event.P(3).py();
                py_tot[1] = proc.event.P(4).py();
                py_tot[2] = proc.event.P(5).py();
                pz_tot[0] = proc.event.P(3).pz();
                pz_tot[1] = proc.event.P(4).pz();
                pz_tot[2] = proc.event.P(5).pz();
                E_tot[0]  = proc.event.P(3).e();
                E_tot[1]  = proc.event.P(4).e();
                E_tot[2]  = proc.event.P(5).e();
                if (info.PrintROOT == 1) ttot->Fill();
                if (info.PrintLHE == 1) ds.writeEvent();
                if (xwgtup > 0.0) iVirtPWt1++;
                if (xwgtup < 0.0) iVirtNWt1++;
                } else {
                if (xwgtup > 0.0) iVirtPWtN1++;
                if (xwgtup < 0.0) iVirtNWtN1++;
                }
            }
    //--------------------------------------------------------------------------------------------------
            else if (rBr <= Br[1] && !info.Iborn && info.Iqed) {
                FoamHard->MakeEvent();   // generate Hard MC event
                FoamHard->GetMCvect(MCvectHard);
                MCwt = FoamHard->GetMCwt();
                xwgtup = proc.event.weight();
                if (MCwt == 1.0) {
                weight_tot[0] = xwgtup;
                px_tot[0] = proc.event.P(3).px();
                px_tot[1] = proc.event.P(4).px();
                px_tot[2] = proc.event.P(5).px();
                py_tot[0] = proc.event.P(3).py();
                py_tot[1] = proc.event.P(4).py();
                py_tot[2] = proc.event.P(5).py();
                pz_tot[0] = proc.event.P(3).pz();
                pz_tot[1] = proc.event.P(4).pz();
                pz_tot[2] = proc.event.P(5).pz();
                E_tot[0]  = proc.event.P(3).e();
                E_tot[1]  = proc.event.P(4).e();
                E_tot[2]  = proc.event.P(5).e();
                weight_hardvar[0] = xwgtup;
                for (int i = 0; i < 5; i++) hardvar[i] = MCvectHard[i];
                if (info.PrintLHE == 1) ds.writeEvent();
                if (info.PrintROOT == 1) ttot->Fill();
                if (info.PrintHard == 1) thardvar->Fill();
                if (xwgtup > 0.0) iHardPWt1++;
                if (xwgtup < 0.0) iHardNWt1++;
                } else {
                if (xwgtup > 0.0) iHardPWtN1++;
                if (xwgtup < 0.0) { iHardNWtN1++; /*MCwt = -MCwt;*/ }
                }
            }
//--------------------------------------------------------------------------------------------------
            else if (rBr <= Br[2] && !info.Iborn && info.Iqedll) {
                FoamLL11->MakeEvent();   // generate LL11 MC event
                FoamLL11->GetMCvect(MCvectLL11);
                MCwt = FoamLL11->GetMCwt();
                xwgtup = proc.event.weight();
                if (MCwt == 1.0) {
                weight_tot[0] = xwgtup;
                px_tot[0] = proc.event.P(3).px();
                px_tot[1] = proc.event.P(4).px();
                px_tot[2] = proc.event.P(5).px();
                py_tot[0] = proc.event.P(3).py();
                py_tot[1] = proc.event.P(4).py();
                py_tot[2] = proc.event.P(5).py();
                pz_tot[0] = proc.event.P(3).pz();
                pz_tot[1] = proc.event.P(4).pz();
                pz_tot[2] = proc.event.P(5).pz();
                E_tot[0]  = proc.event.P(3).e();
                E_tot[1]  = proc.event.P(4).e();
                E_tot[2]  = proc.event.P(5).e();
                if (info.PrintROOT == 1) ttot->Fill();
                if (info.PrintLHE == 1) ds.writeEvent();
                if (xwgtup > 0.0) iLL11PWt1++;
                if (xwgtup < 0.0) iLL11NWt1++;
                } else {
                if (xwgtup > 0.0) iLL11PWtN1++;
                if (xwgtup < 0.0) iLL11NWtN1++;
                }
            }
//--------------------------------------------------------------------------------------------------
            else if (rBr <= Br[3] && !info.Iborn && info.Iqedll) {
                FoamLL22->MakeEvent();   // generate LL22 MC event
                FoamLL22->GetMCvect(MCvectLL22);
                MCwt = FoamLL22->GetMCwt();
                xwgtup = proc.event.weight();
                if (MCwt == 1.0) {
                weight_tot[0] = xwgtup;
                px_tot[0] = proc.event.P(3).px();
                px_tot[1] = proc.event.P(4).px();
                px_tot[2] = proc.event.P(5).px();
                py_tot[0] = proc.event.P(3).py();
                py_tot[1] = proc.event.P(4).py();
                py_tot[2] = proc.event.P(5).py();
                pz_tot[0] = proc.event.P(3).pz();
                pz_tot[1] = proc.event.P(4).pz();
                pz_tot[2] = proc.event.P(5).pz();
                E_tot[0]  = proc.event.P(3).e();
                E_tot[1]  = proc.event.P(4).e();
                E_tot[2]  = proc.event.P(5).e();
                if (info.PrintROOT == 1) ttot->Fill();
                if (info.PrintLHE == 1) ds.writeEvent();
                if (xwgtup > 0.0) iLL22PWt1++;
                if (xwgtup < 0.0) iLL22NWt1++;
                } else {
                if (xwgtup > 0.0) iLL22PWtN1++;
                if (xwgtup < 0.0) iLL22NWtN1++;
                }
            }
//--------------------------------------------------------------------------------------------------
            else if (!info.Iborn && (info.Iqedll >1) && (info.Iqedll !=3) && (info.Iqedll !=4)) {
                FoamLL12->MakeEvent();   // generate LL12 MC event
                FoamLL12->GetMCvect(MCvectLL12);
                MCwt = FoamLL12->GetMCwt();
                xwgtup = proc.event.weight();
                if (MCwt == 1.0) {
                weight_tot[0] = xwgtup;
                px_tot[0] = proc.event.P(3).px();
                px_tot[1] = proc.event.P(4).px();
                px_tot[2] = proc.event.P(5).px();
                py_tot[0] = proc.event.P(3).py();
                py_tot[1] = proc.event.P(4).py();
                py_tot[2] = proc.event.P(5).py();
                pz_tot[0] = proc.event.P(3).pz();
                pz_tot[1] = proc.event.P(4).pz();
                pz_tot[2] = proc.event.P(5).pz();
                E_tot[0]  = proc.event.P(3).e();
                E_tot[1]  = proc.event.P(4).e();
                E_tot[2]  = proc.event.P(5).e();
                if (info.PrintROOT == 1) ttot->Fill();
                if (info.PrintLHE == 1) ds.writeEvent();
                if (xwgtup > 0.0) iLL12PWt1++;
                if (xwgtup < 0.0) iLL12NWt1++;
                } else {
                if (xwgtup > 0.0) iLL12PWtN1++;
                if (xwgtup < 0.0) iLL12NWtN1++;
                }
            }
        }
        ftot->Write();
        fhardvar->Write();
        timing.stopClock();
        timing.ReportClock("Total generation");
//--------------------------------------------------------------------------------------------------
        output_brn << "</LesHouchesEvents>" << endl;
        output_tot << "</LesHouchesEvents>" << endl;
        output_brn.close();
        output_tot.close();
    }
//--------------------------------------------------------------------------------------------------
    double BornF = 1.0;
    double VirtF = 1.0;
    double HardF = 1.0;
    double LL11F = 1.0;
    double LL22F = 1.0;
    double LL12F = 1.0;
    if ((iBornPWt1+iBornNWt1+iBornPWtN1+iBornNWtN1) > 0) BornF = (double) (iBornPWt1-iBornNWt1+iBornPWtN1-iBornNWtN1)/(iBornPWt1+iBornNWt1+iBornPWtN1+iBornNWtN1);
    if ((iVirtPWt1+iVirtNWt1+iVirtPWtN1+iVirtNWtN1) > 0) VirtF = (double) (iVirtPWt1-iVirtNWt1+iVirtPWtN1-iVirtNWtN1)/(iVirtPWt1+iVirtNWt1+iVirtPWtN1+iVirtNWtN1);
    if ((iHardPWt1+iHardNWt1+iHardPWtN1+iHardNWtN1) > 0) HardF = (double) (iHardPWt1-iHardNWt1+iHardPWtN1-iHardNWtN1)/(iHardPWt1+iHardNWt1+iHardPWtN1+iHardNWtN1);
    if ((iLL11PWt1+iLL11NWt1+iLL11PWtN1+iLL11NWtN1) > 0) LL11F = (double) (iLL11PWt1-iLL11NWt1+iLL11PWtN1-iLL11NWtN1)/(iLL11PWt1+iLL11NWt1+iLL11PWtN1+iLL11NWtN1);
    if ((iLL22PWt1+iLL22NWt1+iLL22PWtN1+iLL22NWtN1) > 0) LL22F = (double) (iLL22PWt1-iLL22NWt1+iLL22PWtN1-iLL22NWtN1)/(iLL22PWt1+iLL22NWt1+iLL22PWtN1+iLL22NWtN1);
    if ((iLL12PWt1+iLL12NWt1+iLL12PWtN1+iLL12NWtN1) > 0) LL12F = (double) (iLL12PWt1-iLL12NWt1+iLL12PWtN1-iLL12NWtN1)/(iLL12PWt1+iLL12NWt1+iLL12PWtN1+iLL12NWtN1);
//--------------------------------------------------------------------------------------------------
    cout << "====== Events generated, entering Finalize" << '\n';
    Double_t IntNorm, Errel;
    FoamBorn->Finalize(IntNorm, Errel);
    double SigmaBornAbs(0.),SigmaVirtAbs(0.),SigmaHardAbs(0.),SigmaLL11Abs(0.),SigmaLL22Abs(0.),SigmaLL12Abs(0.);
    double ErrBorn(0.),ErrVirt(0.),ErrHard(0.),ErrLL11(0.),ErrLL22(0.),ErrLL12(0.);
    double aveWt,wtMax,sigma;
//--------------------------------------------------------------------------------------------------
    FoamBorn->GetIntegMC(MCresult, MCerror);  // get MC integral
    info.SigmaBorn = BornF*MCresult;
    ErrBorn = MCerror;
    SigmaBornAbs = MCresult;
//--------------------------------------------------------------------------------------------------
    if (bVirt) {
        FoamVirt->Finalize(IntNorm, Errel);
        FoamVirt->GetIntegMC(MCresult, MCerror);  // get MC integral
        info.SigmaVirt = VirtF*MCresult;
        ErrVirt = MCerror;
        SigmaVirtAbs = MCresult;
    }
//--------------------------------------------------------------------------------------------------
    if (bHard) {
        FoamHard->Finalize(IntNorm, Errel);
        FoamHard->GetIntegMC(MCresult, MCerror);  // get MC integral
        FoamHard->GetWtParams(0.0001,aveWt,wtMax,sigma);
        info.SigmaHard = HardF*MCresult;
        ErrHard = MCerror;
        SigmaHardAbs = MCresult;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && info.Iqedll) {
        FoamLL11->Finalize(IntNorm, Errel);
        FoamLL11->GetIntegMC(MCresult, MCerror);  // get MC integral
        info.SigmaLL11 = LL11F*MCresult;
        ErrLL11 = MCerror;
        SigmaLL11Abs = MCresult;
        delete [] MCvectLL11;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && info.Iqedll) {
        FoamLL22->Finalize(IntNorm, Errel);
        FoamLL22->GetIntegMC(MCresult, MCerror);  // get MC integral
        info.SigmaLL22 = LL22F*MCresult;
        ErrLL22 = MCerror;
        SigmaLL22Abs = MCresult;
        delete [] MCvectLL22;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && (info.Iqedll >1) && (info.Iqedll !=3) && (info.Iqedll !=4)) {
        FoamLL12->Finalize(IntNorm, Errel);
        FoamLL12->GetIntegMC(MCresult, MCerror);  // get MC integral
        info.SigmaLL12 = LL12F*MCresult;
        ErrLL12 = MCerror;
        SigmaLL12Abs = MCresult;
        delete [] MCvectLL12;
    }
//--------------------------------------------------------------------------------------------------
    info.SigmaTot = info.SigmaVirt+info.SigmaHard+info.SigmaLL11+info.SigmaLL22+info.SigmaLL12;
    double SigmaDel = info.SigmaVirt+info.SigmaHard+info.SigmaLL11+info.SigmaLL22+info.SigmaLL12-info.SigmaBorn;
    double SigmaTotAbs = SigmaVirtAbs+SigmaHardAbs+SigmaLL11Abs+SigmaLL22Abs+SigmaLL12Abs;
    double ErrTot = sqrt(ErrVirt*ErrVirt+ErrHard*ErrHard+ErrLL11*ErrLL11+ErrLL22*ErrLL22+ErrLL12*ErrLL12);
    double ErrDel = sqrt(ErrVirt*ErrVirt+ErrHard*ErrHard+ErrLL11*ErrLL11+ErrLL22*ErrLL22+ErrLL12*ErrLL12+ErrBorn*ErrBorn);
    double Delta = SigmaDel/info.SigmaBorn;
    double ErrDelta = abs(Delta)*sqrt(pow2(ErrDel/SigmaDel)+pow2(ErrBorn/info.SigmaBorn));
    ofstream sigma_brn;
    ofstream sigma_tot;
    sigma_brn.open(bornNameDat);
    sigma_tot.open(loopNameDat);
    sigma_brn << setprecision(10) << SigmaBornAbs << '\n';
    sigma_tot << setprecision(10) << SigmaTotAbs << '\n';
    sigma_brn.close();
    sigma_tot.close();
//--------------------------------------------------------------------------------------------------
    delete [] MCvectBorn;
    delete [] MCvectVirt;
    delete [] MCvectHard;
//--------------------------------------------------------------------------------------------------
    cout << "================================================================" << '\n';
    cout << "        MCresult (Born)  = " << info.SigmaBorn << " +- " << ErrBorn << " RelErr = " << ErrBorn/info.SigmaBorn << '\n';
	cout << "        MCresult (B+S+V) = " << info.SigmaVirt << " +- " << ErrVirt << " RelErr = " << ErrVirt/info.SigmaVirt << '\n';
	cout << "        MCresult (Hard)  = " << info.SigmaHard << " +- " << ErrHard << " RelErr = " << ErrHard/info.SigmaHard << '\n';
    cout << "        MCresult (LL11)  = " << info.SigmaLL11 << " +- " << ErrLL11 << " RelErr = " << ErrLL11/info.SigmaLL11 << '\n';
    cout << "        MCresult (LL22)  = " << info.SigmaLL22 << " +- " << ErrLL22 << " RelErr = " << ErrLL22/info.SigmaLL22 << '\n';
    cout << "        MCresult (LL12)  = " << info.SigmaLL12 << " +- " << ErrLL12 << " RelErr = " << ErrLL12/info.SigmaLL12 << '\n';
    cout << "        MCresult (Tot)   = " << info.SigmaTot << " +- " << ErrTot << " RelErr = " << ErrTot/info.SigmaTot << '\n';
    cout << "        MCresult (Delta) = " << Delta << " +- " << ErrDelta << " RelErr = " << ErrDelta/Delta << '\n';
    cout << "================================================================" << '\n';
    cout << "WtMax(for 1e-4 accuracy goal) = "<<wtMax<< '\n';
	cout << "================================================================" << '\n';
    cout << "Total number of Born events:  " << iBornPWt1 + iBornPWtN1 << " pos. " << iBornNWt1 + iBornNWtN1 << " neg." << '\n';
    cout << "Total number of B+S+V events: " << iVirtPWt1 + iVirtPWtN1 << " pos. " << iVirtNWt1 + iVirtNWtN1 << " neg." << '\n';
    cout << "Total number of Hard events:  " << iHardPWt1 + iHardPWtN1 << " pos. " << iHardNWt1 + iHardNWtN1 << " neg." << '\n';
    cout << "Total number of LL11 events:  " << iLL11PWt1 + iLL11PWtN1 << " pos. " << iLL11NWt1 + iLL11NWtN1 << " neg." << '\n';
    cout << "Total number of LL22 events:  " << iLL22PWt1 + iLL22PWtN1 << " pos. " << iLL22NWt1 + iLL22NWtN1 << " neg." << '\n';
    cout << "Total number of LL12 events:  " << iLL12PWt1 + iLL12PWtN1 << " pos. " << iLL12NWt1 + iLL12NWtN1 << " neg." << '\n';
    cout << "================================================================" << '\n';
    cout << "Number of Born events with Wt > 1:  " << iBornPWtN1 << " pos. " << iBornNWtN1 << " neg." << '\n';
    cout << "Number of B+S+V events with Wt > 1: " << iVirtPWtN1 << " pos. " << iVirtNWtN1 << " neg." << '\n';
    cout << "Number of Hard events with Wt > 1:  " << iHardPWtN1 << " pos. " << iHardNWtN1 << " neg." << '\n';
    cout << "Number of LL11 events with Wt > 1:  " << iLL11PWtN1 << " pos. " << iLL11NWtN1 << " neg." << '\n';
    cout << "Number of LL22 events with Wt > 1:  " << iLL22PWtN1 << " pos. " << iLL22NWtN1 << " neg." << '\n';
    cout << "Number of LL12 events with Wt > 1:  " << iLL12PWtN1 << " pos. " << iLL12NWtN1 << " neg." << '\n';
    cout << "================================================================" << '\n';
    cout << "Relative number of Born events with Wt > 1:  " << (double) (iBornPWtN1+iBornNWtN1)/(iBornPWt1+iBornNWt1+iBornPWtN1+iBornNWtN1) << '\n';
    cout << "Relative number of B+S+V events with Wt > 1: " << (double) (iVirtPWtN1+iVirtNWtN1)/(iVirtPWt1+iVirtNWt1+iVirtPWtN1+iVirtNWtN1) << '\n';
    cout << "Relative number of Hard events with Wt > 1:  " << (double) (iHardPWtN1+iHardNWtN1)/(iHardPWt1+iHardNWt1+iHardPWtN1+iHardNWtN1) << '\n';
    cout << "Relative number of LL11 events with Wt > 1:  " << (double) (iLL11PWtN1+iLL11NWtN1)/(iLL11PWt1+iLL11NWt1+iLL11PWtN1+iLL11NWtN1) << '\n';
    cout << "Relative number of LL22 events with Wt > 1:  " << (double) (iLL22PWtN1+iLL22NWtN1)/(iLL22PWt1+iLL22NWt1+iLL22PWtN1+iLL22NWtN1) << '\n';
    cout << "Relative number of LL12 events with Wt > 1:  " << (double) (iLL12PWtN1+iLL12NWtN1)/(iLL12PWt1+iLL12NWt1+iLL12PWtN1+iLL12NWtN1) << '\n';
    cout << "***** End of Program *****" << '\n';
}

void ReneSANCe_ee::generateEventsNB()
{
    TFoam *FoamBorn,*FoamLoop;
    TFoamIntegrand *Density0,*Density3;
    TFile *fileBorn,*fileLoop;
    timing.startClock();
    if (info.IEXPLBORN) {
	FoamBorn = new TFoam("FoamBorn");   // Create Simulator
	Density0 = new Integrand(info, proc, 0);   // Create integrand function
	FoamBorn->SetkDim(proc.kDimBorn);
	FoamBorn->SetnCells(info.nCellsBorn);
	FoamBorn->SetnSampl(info.nSamplBorn);
	FoamBorn->SetnBin(info.nBinBorn);
	FoamBorn->SetEvPerBin(info.EvPerBinBorn);
	FoamBorn->SetOptRej(info.OptRej);
	FoamBorn->SetOptDrive(info.OptDrive);
	FoamBorn->SetMaxWtRej(info.MaxWtRejBorn);
	FoamBorn->SetChat(info.Chat);
	FoamBorn->SetRho(Density0);
	FoamBorn->SetPseRan(PseRan);
	FoamBorn->SetInhiDiv(1, 1);
	FoamBorn->Initialize();
	TFile RootBornFile("FoamBorn.root","RECREATE","Born");
	FoamBorn->Write("FoamBorn");    // Writing Foam on the disk
    } else {
	fileBorn = new TFile("FoamBorn.root"); // connect disk file
	FoamBorn = (TFoam*)fileBorn->Get("FoamBorn"); // find object
	Density0 = new Integrand(info, proc, 0);   // Create integrand function
	FoamBorn->SetPseRan(PseRan);
	FoamBorn->SetRho(Density0);
	FoamBorn->SetMaxWtRej(info.MaxWtRejBorn);
	FoamBorn->LinkCells(); // restore pointers of the binary tree of cells
	FoamBorn->GetTotDim();
    }
    timing.stopClock();
    timing.ReportClock("Born initialization");
    if (!info.Iborn && (info.Iew || info.Iqed)) {
	timing.startClock();
	if (info.IEXPLHARD) {
	    FoamLoop = new TFoam("FoamLoop");   // Create Simulator
	    Density3 = new Integrand(info, proc, 3);   // Create integrand function
	    FoamLoop->SetkDim(proc.kDimHard+1);
	    FoamLoop->SetnCells(info.nCellsHard);
	    FoamLoop->SetnSampl(info.nSamplHard);
	    FoamLoop->SetnBin(info.nBinHard);
	    FoamLoop->SetEvPerBin(info.EvPerBinHard);
	    FoamLoop->SetOptRej(info.OptRej);
	    FoamLoop->SetOptDrive(info.OptDrive);
	    FoamLoop->SetMaxWtRej(info.MaxWtRejHard);
	    FoamLoop->SetChat(info.Chat);
	    FoamLoop->SetRho(Density3);
	    FoamLoop->SetPseRan(PseRan);
	    FoamLoop->SetInhiDiv(5,1);
	    double xDiv[1];
	    xDiv[0]=0.5;
	    FoamLoop->SetXdivPRD(5,1,xDiv);
	    FoamLoop->Initialize();
	    TFile RootVirtFile("FoamLoop.root","RECREATE","Loop");
	    FoamLoop->Write("FoamLoop");    // Writing Foam on the disk
	} else {
	    fileLoop = new TFile("FoamLoop.root"); // connect disk file
	    FoamLoop = (TFoam*)fileLoop->Get("FoamLoop"); // find object
	    Density3 = new Integrand(info, proc, 3);   // Create integrand function
	    FoamLoop->SetPseRan(PseRan);
	    FoamLoop->SetRho(Density3);
	    FoamLoop->SetMaxWtRej(info.MaxWtRejHard);
	    FoamLoop->LinkCells(); // restore pointers of the binary tree of cells
	    FoamLoop->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("loop initialization");
    }
    timing.startClock();
    cout << "====== Initialization done, entering MC loop" << '\n';
//--------------------------------------------------------------------------------------------------
    double *MCvectBorn = new double[proc.kDimBorn];   // vector generated in the MC run
    double *MCvectLoop = new double[proc.kDimHard+1];   // vector generated in the MC run
//--------------------------------------------------------------------------------------------------
    int iBornPWt1  = 0, iBornNWt1  = 0;
    int iBornPWtN1 = 0, iBornNWtN1 = 0;
    int iLoopPWt1  = 0, iLoopNWt1  = 0;
    int iLoopPWtN1 = 0, iLoopNWtN1 = 0;
//--------------------------------------------------------------------------------------------------
    mkdir(info.outputDir.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
    output_brn.open(info.outputDir+"/SANCeeLO.lhe");
    output_tot.open(info.outputDir+"/SANCeeNLO.lhe");
//--------------------------------------------------------------------------------------------------
	ds.setOutput(&output_brn);
	ds.start();
//--------------------------------------------------------------------------------------------------
    TFile *fborn = new TFile((info.outputDir+"/SANCeeLO.root").c_str(),"recreate");
    TTree *tborn = new TTree("events_born","born events");
    Double_t weight_born[1], px_born[3], py_born[3], pz_born[3], E_born[3];
    tborn->Branch("weight_born", weight_born, "weight_born[1]/D");
    tborn->Branch("px_born", px_born, "px_born[3]/D");
    tborn->Branch("py_born", py_born, "py_born[3]/D");
    tborn->Branch("pz_born", pz_born, "pz_born[3]/D");
    tborn->Branch("E_born", E_born, "E_born[3]/D");
//--------------------------------------------------------------------------------------------------
    for (loop = 0; loop < info.NevBorn; loop++) {
	FoamBorn->MakeEvent();   // generate Born MC event
	FoamBorn->GetMCvect(MCvectBorn);
	MCwt = FoamBorn->GetMCwt();
	xwgtup = proc.event.weight();
	if (MCwt == 1.0) {
	    weight_born[0] = xwgtup;
	    px_born[0] = proc.event.P(3).px();
	    px_born[1] = proc.event.P(4).px();
	    px_born[2] = proc.event.P(5).px();
	    py_born[0] = proc.event.P(3).py();
	    py_born[1] = proc.event.P(4).py();
	    py_born[2] = proc.event.P(5).py();
	    pz_born[0] = proc.event.P(3).pz();
	    pz_born[1] = proc.event.P(4).pz();
	    pz_born[2] = proc.event.P(5).pz();
	    E_born[0]  = proc.event.P(3).e();
	    E_born[1]  = proc.event.P(4).e();
	    E_born[2]  = proc.event.P(5).e();
	    if (info.PrintLHE == 1) ds.writeEvent();
	    if (info.PrintROOT == 1) tborn->Fill();
	    if (xwgtup > 0.0) iBornPWt1++;
	    if (xwgtup < 0.0) iBornNWt1++;
	} else {
	    if (xwgtup > 0.0) iBornPWtN1++;
	    if (xwgtup < 0.0) iBornNWtN1++;
	}
    }
    fborn->Write();
    timing.stopClock();
    timing.ReportClock("Born generation");
//--------------------------------------------------------------------------------------------------
	ds.setOutput(&output_tot);
	ds.start();
//--------------------------------------------------------------------------------------------------
    TFile *ftot = new TFile((info.outputDir+"/SANCeeNLO.root").c_str(),"recreate");
    TTree *ttot = new TTree("events_tot","tot events");
    Double_t weight_tot[1], px_tot[3], py_tot[3], pz_tot[3], E_tot[3];
    ttot->Branch("weight_tot", weight_tot, "weight_tot[1]/D");
    ttot->Branch("px_tot", px_tot, "px_tot[3]/D");
    ttot->Branch("py_tot", py_tot, "py_tot[3]/D");
    ttot->Branch("pz_tot", pz_tot, "pz_tot[3]/D");
    ttot->Branch("E_tot", E_tot, "E_tot[3]/D");
//--------------------------------------------------------------------------------------------------
    timing.startClock();
    for (loop = 0; loop < info.NevTot; loop++) {
	FoamLoop->MakeEvent();   // generate Hard MC event
	FoamLoop->GetMCvect(MCvectLoop);
	MCwt = FoamLoop->GetMCwt();
	xwgtup = proc.event.weight();
	if (MCwt == 1.0) {
	    weight_tot[0] = xwgtup;
	    px_tot[0] = proc.event.P(3).px();
	    px_tot[1] = proc.event.P(4).px();
	    px_tot[2] = proc.event.P(5).px();
	    py_tot[0] = proc.event.P(3).py();
	    py_tot[1] = proc.event.P(4).py();
	    py_tot[2] = proc.event.P(5).py();
	    pz_tot[0] = proc.event.P(3).pz();
	    pz_tot[1] = proc.event.P(4).pz();
	    pz_tot[2] = proc.event.P(5).pz();
	    E_tot[0]  = proc.event.P(3).e();
	    E_tot[1]  = proc.event.P(4).e();
	    E_tot[2]  = proc.event.P(5).e();
	    if (info.PrintLHE == 1) ds.writeEvent();
	    if (info.PrintROOT == 1) ttot->Fill();
	    if (xwgtup > 0.0) iLoopPWt1++;
	    if (xwgtup < 0.0) iLoopNWt1++;
	} else {
	    if (xwgtup > 0.0) iLoopPWtN1++;
	    if (xwgtup < 0.0) iLoopNWtN1++;
        }
    }
    ftot->Write();
    timing.stopClock();
    timing.ReportClock("Total generation");
//--------------------------------------------------------------------------------------------------
    output_brn << "</LesHouchesEvents>" << endl;
    output_tot << "</LesHouchesEvents>" << endl;
    output_brn.close();
    output_tot.close();
//--------------------------------------------------------------------------------------------------
    double BornF = 1.;
    double HardF = 1.;
    if ((iBornPWt1+iBornNWt1+iBornPWtN1+iBornNWtN1) > 0) BornF = (double) (iBornPWt1-iBornNWt1+iBornPWtN1-iBornNWtN1)/(iBornPWt1+iBornNWt1+iBornPWtN1+iBornNWtN1);
    if ((iLoopPWt1+iLoopNWt1+iLoopPWtN1+iLoopNWtN1) > 0) HardF = (double) (iLoopPWt1-iLoopNWt1+iLoopPWtN1-iLoopNWtN1)/(iLoopPWt1+iLoopNWt1+iLoopPWtN1+iLoopNWtN1);
//--------------------------------------------------------------------------------------------------
    cout << "====== Events generated, entering Finalize" << '\n';
    Double_t IntNorm, Errel;
    double aveWt,wtMax,sigma;
    FoamBorn->Finalize(IntNorm, Errel);
    FoamLoop->Finalize(IntNorm, Errel);
    FoamLoop->GetWtParams(0.0001,aveWt,wtMax,sigma);
//--------------------------------------------------------------------------------------------------
    FoamBorn->GetIntegMC(MCresult, MCerror);  // get MC integral
    info.SigmaBorn = BornF*MCresult;
    double ErrBorn = MCerror;
    double SigmaBornAbs = MCresult;
    cout << "================================================================" << '\n';
    cout << "        MCresult (Born) = " << info.SigmaBorn << " +- " << ErrBorn << " RelErr = " << ErrBorn/info.SigmaBorn << '\n';
//--------------------------------------------------------------------------------------------------
    FoamLoop->GetIntegMC(MCresult, MCerror);  // get MC integral
    info.SigmaHard = HardF*MCresult;
    double ErrHard = MCerror;
    double SigmaHardAbs = MCresult;
    cout << "        MCresult (Hard) = " << info.SigmaHard << " +- " << ErrHard << " RelErr = " << ErrHard/info.SigmaHard << '\n';
//--------------------------------------------------------------------------------------------------
    info.SigmaTot = info.SigmaHard;
    double SigmaDel = info.SigmaHard-info.SigmaBorn;
    double SigmaTotAbs = SigmaHardAbs;
    double ErrTot = sqrt(ErrHard*ErrHard);
    double ErrDel = sqrt(ErrHard*ErrHard+ErrBorn*ErrBorn);
    double Delta = SigmaDel/info.SigmaBorn;
    double ErrDelta = Delta*sqrt(ErrDel/SigmaDel*ErrDel/SigmaDel+ErrBorn/info.SigmaBorn*ErrBorn/info.SigmaBorn);
    cout << "        MCresult (Tot)  = " << info.SigmaTot << " +- " << ErrTot << " RelErr = " << ErrTot/info.SigmaTot << '\n';
    cout << "        MCresult (Delta)  = " << Delta << " +- " << ErrDelta << " RelErr = " << ErrDelta/Delta << '\n';
    cout << "================================================================" << '\n';
    cout << "WtMax(esp= 0.0001) = "<<wtMax<< '\n';
    ofstream sigma_brn;
    ofstream sigma_tot;
    sigma_brn.open(info.outputDir+"/SANCeeLO.dat");
    sigma_tot.open(info.outputDir+"/SANCeeNLO.dat");
    sigma_brn << setprecision(10) << SigmaBornAbs << '\n';
    sigma_tot << setprecision(10) << SigmaTotAbs << '\n';
    sigma_brn.close();
    sigma_tot.close();
//--------------------------------------------------------------------------------------------------
    delete [] MCvectBorn;
    delete [] MCvectLoop;
//--------------------------------------------------------------------------------------------------
    cout << "Total number of Born events: " << iBornPWt1 + iBornPWtN1 << " pos. " << iBornNWt1 + iBornNWtN1 << " neg." << '\n';
    cout << "Total number of Loop events: " << iLoopPWt1 + iLoopPWtN1 << " pos. " << iLoopNWt1 + iLoopNWtN1 << " neg." << '\n';
    cout << "================================================================" << '\n';
    cout << "Number of Born events with Wt > 1: " << iBornPWtN1 << " pos. " << iBornNWtN1 << " neg." << '\n';
    cout << "Number of Hard events with Wt > 1: " << iLoopPWtN1 << " pos. " << iLoopNWtN1 << " neg." << '\n';
    cout << "================================================================" << '\n';
    cout << "Relative number of Born events with Wt > 1: " << (double) (iBornPWtN1+iBornNWtN1)/(iBornPWt1+iBornNWt1+iBornPWtN1+iBornNWtN1) << '\n';
    cout << "Relative number of Hard events with Wt > 1: " << (double) (iLoopPWtN1+iLoopNWtN1)/(iLoopPWt1+iLoopNWt1+iLoopPWtN1+iLoopNWtN1) << '\n';
    cout << "***** End of Program *****" << '\n';
}
