# This module tries to find the LHAPDF installation on your system.

message(STATUS "Looking for LHAPDF")

set(LHAPDF_FOUND 0)

# try to find LHAPDF in user defined path
find_library(LHAPDF_LIBRARY
    NAMES
        libLHAPDF.so LHAPDF libLHAPDF.a libLHAPDF.la
    PATH_SUFFIXES
        lib lib64
    PATHS
        $ENV{LHAPDF_ROOT}
        $ENV{LHAPDF_DIR}
        ${LHAPDF_DIR}
        /opt/LHAPDF
        /usr/local
        /usr
        ~/local
        ~/local/LHAPDF
    DOC "Specify the LHAPDF library here."
)

#if not found try to use lhapdf-config tool
IF(${LHAPDF_LIBRARY} MATCHES "LHAPDF_LIBRARY-NOTFOUND")
	find_program(LHAPDF_CONFIG_EXECUTABLE NAMES lhapdf-config)
	IF(${LHAPDF_CONFIG_EXECUTABLE} MATCHES "LHAPDF_CONFIG_EXECUTABLE-NOTFOUND")
		message(STATUS "Looking for LHAPDF... - lhapdf-config executable not found")
	ELSE(${LHAPDF_CONFIG_EXECUTABLE} MATCHES "LHAPDF_CONFIG_EXECUTABLE-NOTFOUND")
		message(STATUS "Looking for LHAPDF... - using lhapdf-config executable")
		execute_process(COMMAND ${LHAPDF_CONFIG_EXECUTABLE} --prefix OUTPUT_VARIABLE LHAPDF_DIR OUTPUT_STRIP_TRAILING_WHITESPACE)
		find_library(LHAPDF_LIBRARY
		    NAMES
                libLHAPDF.so LHAPDF libLHAPDF.a libLHAPDF.la
            PATH_SUFFIXES
                lib lib64
		    PATHS
                ${LHAPDF_DIR}
		    NO_DEFAULT_PATH
		)
	ENDIF(${LHAPDF_CONFIG_EXECUTABLE} MATCHES "LHAPDF_CONFIG_EXECUTABLE-NOTFOUND")
ENDIF(${LHAPDF_LIBRARY} MATCHES "LHAPDF_LIBRARY-NOTFOUND")

IF(NOT ${LHAPDF_LIBRARY} MATCHES "LHAPDF_LIBRARY-NOTFOUND")
	FIND_PATH(LHAPDF_INCLUDE
	    LHAPDF/LHAPDF.h
	    PATHS
	        $ENV{LHAPDF_ROOT}/include
	        $ENV{LHAPDF_DIR}/include
	        ${LHAPDF_DIR}/include
	        /usr/include
	        /usr/local/include
            ~/local/include
            ~/local/LHAPDF/include
	)
ENDIF(NOT ${LHAPDF_LIBRARY} MATCHES "LHAPDF_LIBRARY-NOTFOUND")

# final printout.
IF((${LHAPDF_LIBRARY} MATCHES "LHAPDF_LIBRARY-NOTFOUND") OR (${LHAPDF_INCLUDE} MATCHES "LHAPDF_INCLUDE-NOTFOUND"))
    set(LHAPDF_LIBRARY "" CACHE FILEPATH "" FORCE)
    set(LHAPDF_FOUND 0 )
ELSE((${LHAPDF_LIBRARY} MATCHES "LHAPDF_LIBRARY-NOTFOUND") OR (${LHAPDF_INCLUDE} MATCHES "LHAPDF_INCLUDE-NOTFOUND"))
	message(STATUS "Looking for LHAPDF... - found " ${LHAPDF_LIBRARY} )
    message(STATUS "Looking for LHAPDF... - found " ${LHAPDF_INCLUDE} )
# Fix for cms release
    set(LHAPDF_LIBRARY ${LHAPDF_LIBRARY} -lz)
    set(LHAPDF_FOUND 1 )
#	include_directories(${LHAPDF_INCLUDE})
ENDIF((${LHAPDF_LIBRARY} MATCHES "LHAPDF_LIBRARY-NOTFOUND") OR (${LHAPDF_INCLUDE} MATCHES "LHAPDF_INCLUDE-NOTFOUND"))
